import sys
import click
import time
import calendar
import datetime

from anchore.cli.common import anchore_print, anchore_print_err
from anchore import anchore_auth, anchore_feeds
from anchore.anchore_utils import contexts

config = {}

@click.group(name='feeds', short_help='Manage syncing of and subscriptions to Anchore data feeds.')
@click.pass_obj
def feeds(anchore_config):
    global config
    config = anchore_config

    ecode = 0
    emsg = ""
    success = True

    try:
        rc, msg = anchore_feeds.check()
        if not rc:
            anchore_print("initializing feed metadata: ...")
            rc, ret = anchore_feeds.sync_feedmeta()
            if not rc:
                emsg = "could not sync feed metadata from service: " + ret['text']
                success = False

    except Exception as err:
        anchore_print_err('operation failed')
        sys.exit(1)

    if not success:
        anchore_print_err(emsg)
        sys.exit(1)

@feeds.command(name='show', short_help='Show detailed info on a specific feed')
@click.argument('feed')
def show(feed):
    """
    Show detailed feed information

    """
    ecode = 0
    try:
        feedmeta = anchore_feeds.load_anchore_feedmeta()
        if feed in feedmeta:
            result = {}
            groups = feedmeta[feed].get('groups',{}).values()
            result['name'] = feed
            result['access_tier'] = int(feedmeta[feed].get('access_tier'))
            result['description'] = feedmeta[feed].get('description')
            result['groups'] = {}
            if 'subscribed' not in feedmeta[feed]:
                result['subscribed'] = False
            else:
                result['subscribed'] = feedmeta[feed]['subscribed']

            for g in groups:
                result['groups'][g['name']] = {
                    'access_tier': int(g.get('access_tier')),
                    'description': g.get('description'),
                    'last_sync': datetime.datetime.fromtimestamp(g.get('last_update')).isoformat() if 'last_update' in g else 'None'
                }

            anchore_print(result, do_formatting=True)
        else:
            anchore_print_err('Unknown feed name. Valid feeds can be seen withe the "list" command')
            ecode = 1
    except Exception as err:
        anchore_print_err('operation failed')
        ecode = 1

    sys.exit(ecode)

@feeds.command(name='list', short_help="List all feeds.")
@click.option('--showgroups', help='Along with the feed, show all groups within the feed.', is_flag=True)
def list(showgroups):
    """
    Show list of Anchore data feeds.
    """

    ecode = 0
    try:
        result = {}
        subscribed = {}
        available = {}
        unavailable = {}
        current_user_data = contexts['anchore_auth']['user_info']
        feedmeta = anchore_feeds.load_anchore_feedmeta()

        for feed in feedmeta.keys():
            if feedmeta[feed]['subscribed']:
                subscribed[feed] = {}
                subscribed[feed]['description'] = feedmeta[feed]['description']
                if showgroups:
                    subscribed[feed]['groups'] = feedmeta[feed]['groups'].keys()

            else:
                if current_user_data:
                    tier = int(current_user_data['tier'])
                else:
                    tier = 0

                if int(feedmeta[feed]['access_tier']) > tier:
                    collection = unavailable
                else:
                    collection = available

                collection[feed] = {}

                collection[feed]['description'] = feedmeta[feed]['description']
                if showgroups and collection == available:
                    collection[feed]['groups'] = feedmeta[feed]['groups'].keys()

        if available:
            result['Available'] = available
        if subscribed:
            result['Subscribed'] = subscribed
        if unavailable:
            result['Unavailable/Insufficient Access Tier'] = unavailable

        anchore_print(result, do_formatting=True)
    except Exception as err:
        anchore_print_err('operation failed')
        ecode = 1

    sys.exit(ecode)

@feeds.command(name='sub', short_help="Subscribe to specified feed(s).")
@click.argument('feednames', nargs=-1, metavar='<feedname> <feedname> ...')
def sub(feednames):
    """
    Subscribe to the specified feed(s).
    """

    ecode = 0
    current_user_data = contexts.get('anchore_auth', {}).get('user_info', None)
    if not current_user_data:
        current_user_tier = 0
    else:
        current_user_tier = int(current_user_data['tier'])

    try:
        for feed in feednames:
            rc, msg = anchore_feeds.subscribe_anchore_feed(feed, current_user_tier)
            if not rc:
                ecode = 1
                anchore_print_err(msg)
            else:
                anchore_print(msg)

    except Exception as err:
        anchore_print_err('operation failed')
        ecode = 1

    sys.exit(ecode)

@feeds.command(name='unsub', short_help="Unsubscribe from specified feed(s).")
@click.argument('feednames', nargs=-1, metavar='<feedname> <feedname> ...')
@click.option('--delete', help='Delete all feed data after unsubscribing', is_flag=True)
@click.option('--dontask', help='Used with --delete, will not prompt before deleting all feed data', is_flag=True)
def unsub(feednames, delete, dontask):
    """
    Unsubscribe from the specified feed(s).
    """

    ecode = 0
    try:
        for feed in feednames:
            rc, msg = anchore_feeds.unsubscribe_anchore_feed(feed)
            if not rc:
                ecode = 1
                anchore_print_err(msg)
            else:
                anchore_print(msg)
                if delete:
                    dodelete = False
                    if dontask:
                        dodelete = True
                    else:
                        try:
                            answer = raw_input("Really delete feed data ("+str(feed)+"'? (y/N)")
                        except:
                            answer = "n"
                        if 'y' == answer.lower():
                            dodelete = True
                        else:
                            anchore_print(str(feed) + ": skipping delete.")

                    if dodelete:
                        anchore_print(str(feed) + ": deleting feed.")
                        rc = anchore_feeds.delete_anchore_feed(feed)

    except Exception as err:
        anchore_print_err('operation failed')
        ecode = 1

    sys.exit(ecode)

@feeds.command(name='sync', short_help="Sync (download) latest data for all subscribed feeds from the Anchore service.")
@click.option('--since', help='Force a feed sync from the given timestamp to today.', metavar='<unix timestamp>')
@click.option('--do-compact', help='After syncing, process feed data to eliminate duplicate entries and store only latest data records', is_flag=True)
def sync(since, do_compact):
    """
    Sync (download) latest data for all subscribed feeds from the Anchore service.
    """

    ecode = 0
    try:
        rc, ret = anchore_feeds.sync_feedmeta()
        if not rc:
            anchore_print_err(ret['text'])
            ecode = 1
        else:
            rc, ret = anchore_feeds.sync_feeds(force_since=since, do_combine=do_compact)
            if not rc:
                anchore_print_err(ret['text'])
                ecode = 1
    except Exception as err:
        anchore_print_err('operation failed')
        ecode = 1

    sys.exit(ecode)



#!/usr/bin/env python

import sys
import os
import re
import json
import anchore.anchore_utils

gate_name = "SUIDDIFF"
triggers = {
    'SUIDMODEDIFF':
    {
        'description':'triggers if file is suid, but mode is different between the image and its base',
        'params':'none'
    },
    'SUIDFILEADD':
    {
        'description':'triggers if the evaluated image has a file that is SUID and the base image does not',
        'params':'none'
    },
    'SUIDFILEDEL':
    {
        'description':'triggers if the base image has a SUID file, but the evaluated image does not',
        'params':'none'
    },
    'SUIDDIFF':
    {
        'description':'triggers if any one of the other events for this gate have triggered',
        'params':'none'
    },
}

try:
    config = anchore.anchore_utils.init_gate_cmdline(sys.argv, gate_name, gate_help=triggers)
except Exception as err:
    print str(err)
    sys.exit(1)

if not config:
    sys.exit(0)

imgid = config['imgid']

try:
    params = config['params']
except:
    params = None

if config['meta']['usertype'] != 'user':
    sys.exit(0)

outlist = list()

imageId = config['imgid']
baseId = config['baseid']

diffdata = anchore.anchore_utils.diff_images(imageId, baseId)
try:
    isdiff = False
    pkgdiffs = diffdata.pop('file_suids', {}).pop('files.suids', {})
    for module_type in pkgdiffs.keys():
        for pkg in pkgdiffs[module_type].keys():
            isdiff = True
            status = pkgdiffs[module_type][pkg]
            if (status == 'VERSION_DIFF'):
                trigger = "SUIDMODEDIFF"
                d = {'id':'-'.join([pkg, trigger]), 'desc':"SUID file mode in container is different from baseline for file - " + pkg}
                #outlist.append("SUIDMODEDIFF SUID file mode in container is different from baseline for file - " + pkg)
                outlist.append(trigger +  " " + json.dumps(d))
            elif (status == 'INIMG_NOTINBASE'):
                trigger = "SUIDFILEADD"
                d = {'id':'-'.join([pkg, trigger]), 'desc':"SUID file has been added to image since base - " + pkg}
                #outlist.append("SUIDFILEADD SUID file has been added to image since base - " + pkg)
                outlist.append(trigger + " " + json.dumps(d))
            elif (status == 'INBASE_NOTINIMG'):
                trigger = "SUIDFILEDEL"
                d = {'id':'-'.join([pkg, trigger]), 'desc':"SUID file has been removed from image since base - " + pkg}
                #outlist.append("SUIDFILEDEL SUID file has been removed from image since base - " + pkg)
                outlist.append(trigger + " " + json.dumps(d))

    if (isdiff):
        outlist.append("SUIDDIFF SUID file manifest is different from image to base")
except Exception as err:
    print "ERROR: running gate " + gate_name + " failed: " + str(err)
    sys.exit(1)

anchore.anchore_utils.save_gate_output(imageId, gate_name, outlist)

sys.exit(0)



#!/usr/bin/env python

import sys
import os
import re
import anchore.anchore_utils

def perform_cve_scan(severity='all'):
    severity = severity
    ret = {}
    cve_ret = {}
    warns = list()
    
    cve_data_context = {}
    
    for imageId in config['images']:
        image = allimages[imageId]
        try:
            cve_data = anchore.anchore_utils.cve_load_data(image, cve_data_context=cve_data_context)
        except ValueError as err:
            warns.append(image.meta['shortId']+" ("+image.get_human_name()+") " + str(err))
            cve_data = None

        earliest_vulnerable = None
        if cve_data:
            cve_results = anchore.anchore_utils.cve_scanimage(cve_data, imageId)

            fresults = {}
            for fid in image.get_familytree():
                fimage = allimages[fid]
                if fimage.is_user() or fimage.is_base() or fid == image.meta['imageId']:
                    if fid == image.meta['imageId']:
                        fcve_results = cve_results.copy()
                    else:
                        fcve_results = anchore.anchore_utils.cve_scanimage(cve_data, fimage)
                    fresults[fid] = fcve_results                
            print "finished scanning"
            if cve_results:
                for cveId in cve_results.keys():
                    for cvepkg in cve_results[cveId]:
                        if severity == 'all' or cvepkg['severity'] == severity:
                            depth=0
                            branch = list()
                            for fid in image.get_familytree():
                                fimage = allimages[fid]
                                if fimage.is_user() or fimage.is_base() or fid == image.meta['imageId']:
                                    fcve_results = fresults[fid]
                                    if cveId in fcve_results.keys():
                                        branch.append(fid)
                                        if not cve_ret:
                                            cve_ret = fcve_results.copy()
                                        cve_ret.update(fcve_results)

                            if len(branch) > 0:
                                if cveId not in ret:
                                    ret[cveId] = list()
                                ret[cveId].append(branch)
        else:
            warns.append(image.meta['shortId']+" ("+image.get_human_name()+") cannot scan image: no CVE data loaded from feed")
                             
    branches = ret
    cve_results = cve_ret

    ret = {}
    for cveId in branches.keys():

        tofix = {}
        torebuild = {}

        for b in branches[cveId]:
            image = allimages[b[0]]

            tofix[''.join(image.get_human_string().split())] = True
            for bk in b[1:]:
                image = allimages[bk]
                torebuild[''.join(image.get_human_string().split())] = True

            total_vuln = len(tofix.keys()) + len(torebuild.keys())

            if not cveId in ret:
                ret[cveId] = list()

            if len(torebuild.keys()) <= 0:
                rebuildstr = 'None'
            else:
                #rebuildstr = ','.join(torebuild.keys())
                rebuildstr = ' '.join(torebuild.keys())

            if len(tofix.keys()) <= 0:
                fixstr = 'None'
            else:
                #fixstr = ','.join(tofix.keys())
                fixstr = ' '.join(tofix.keys())

        if cveId in cve_results:
            for cvepkg in cve_results[cveId]:
                if cvepkg['fixVers'] == 'None':
                    fixpkg = "None"
                else:
                    fixpkg = cvepkg['fixVers']

                outsev = "N/A"
                if 'severity' in cvepkg:
                    outsev = cvepkg['severity']

                outpkg = "N/A"
                if 'pkgName' in cvepkg and 'imageVers' in cvepkg:
                    outpkg = cvepkg['pkgName'] + "-" + cvepkg['imageVers']

                outurl = "N/A"
                if 'url' in cvepkg:
                    outurl = cvepkg['url']

                ret[cveId].append([str(cveId), str(outsev), str(total_vuln), str(outpkg), str(fixpkg), str(fixstr), str(rebuildstr), str(outurl)])

    return(ret, warns)

# main routine

# parse cmdline into config dict
try:
    config = anchore.anchore_utils.init_query_cmdline(sys.argv, "params: <Low|Medium|High|Critical|Unknown|Negligible> <another level> ...\nhelp: use 'all' to scan for all severity levels")
except Exception as err:
    print str(err)
    sys.exit(1)

# this means that no config is available, for example when 'help' is given as a parameter
if not config:
    sys.exit(0)

# handle default case if no params are given
if len(config['params']) <= 0:
    config['params'] = ['all']

# set up output list
outlist = list()
outlist.append(["CVE_ID","Severity","*Total_Affected","Vulnerable_Package", "Fix_Available","Fix_Images","Rebuild_Images", "URL"])

# perform query
allimages = {}
for imageId in config['images']:
    image = anchore.anchore_image.AnchoreImage(imageId, config['anchore_config']['image_data_store'], allimages)

    if imageId not in allimages:
        allimages[imageId] = image
        
#for sev in config['params']:
try:
    #cvedatadir = '/'.join([config['anchore_config']['image_data_store'], '..', 'cve-data'])
    warns = list()
    result = {}
    try:
        #result, warns = perform_cve_scan(cvedatadir, sev)
        result, warns = perform_cve_scan('all')
    except ValueError as err:
        print str(err)
        #sys.exit(1)

    for k in result.keys():
        for kk in result[k]:
            if 'all' in config['params'] or kk[1] in config['params']:
                outlist.append(kk)

except Exception as err:
    print str(err)
    import traceback
    traceback.print_exc()

# save output list as output file for anchore to ingress
anchore.anchore_utils.write_kvfile_fromlist(config['output'], outlist)

if len(warns) > 0:
    anchore.anchore_utils.write_plainfile_fromlist(config['output_warns'], warns)

# cleanup and exit
allimages.clear()
sys.exit(0)



#!/usr/bin/env python

import sys
import os
import shutil
import re
import json
import time
import rpm
import subprocess
import tarfile

import anchore.anchore_utils

analyzer_name = "retrieve_files"

try:
    config = anchore.anchore_utils.init_analyzer_cmdline(sys.argv, analyzer_name)
except Exception as err:
    print str(err)
    sys.exit(1)

imgname = config['imgid']
imgid = config['imgid_full']
outputdir = config['dirs']['outputdir']
unpackdir = config['dirs']['unpackdir']
rootfsdir = '/'.join([unpackdir, 'rootfs'])

#if not os.path.exists(outputdir):
#    os.makedirs(outputdir)

files_to_get = list()
if 'analyzer_config' in config and config['analyzer_config']:
    if 'file_list' in config['analyzer_config']  and type(config['analyzer_config']['file_list']) == list:
        files_to_get = config['analyzer_config']['file_list']

if len(files_to_get) <= 0:
    print "No configuration found in analyzer_config.yaml for analyzer '"+analyzer_name+", skipping"
    sys.exit(0)

outputdata = {}
storefiles = list()
for f in files_to_get:
    thefile = '/'.join([unpackdir, 'rootfs', f])
    if os.path.exists(thefile):
        outputdata[f] = thefile
        storefiles.append(thefile)
    else:
        pass

try:
    anchore.anchore_utils.save_files(imgid, analyzer_name, rootfsdir, storefiles)
except Exception as err:
    print "ERROR: unable to store files - exception: " + str(err)
    outputdata = {}
    
#if not os.path.exists('/'.join([outputdir, "file_cache"])):
#    os.makedirs('/'.join([outputdir, "file_cache"]))

#outputdata = {}
#tar = tarfile.open('/'.join([outputdir, 'file_cache', 'stored_files.tar.gz']), mode='w:gz', format=tarfile.PAX_FORMAT)
#for f in files_to_get:
#    thefile = '/'.join([unpackdir, 'rootfs', f])
#    if os.path.exists(thefile):
#        print "INFO: storing file: " + str(thefile)
#        outputdata[f] = thefile
#        tar.add(thefile, filter=tarfilter)
#    else:
#        print "WARN: could not find file ("+str(thefile)+") in image: skipping retrieve"
#tar.close()

if outputdata:
    ofile = os.path.join(outputdir, 'file_cache.all')
    anchore.anchore_utils.write_kvfile_fromdict(ofile, outputdata)

sys.exit(0)



#!/usr/bin/env python

import sys
import os
import shutil
import re
import json
import time
import rpm
import subprocess

import anchore.anchore_utils

analyzer_name = "package_list"

try:
    config = anchore.anchore_utils.init_analyzer_cmdline(sys.argv, analyzer_name)
except Exception as err:
    print str(err)
    sys.exit(1)

imgname = config['imgid']
imgid = config['imgid_full']
outputdir = config['dirs']['outputdir']
unpackdir = config['dirs']['unpackdir']

meta = anchore.anchore_utils.get_distro_from_path('/'.join([unpackdir, "rootfs"]))
distrodict = anchore.anchore_utils.get_distro_flavor(meta['DISTRO'], meta['DISTROVERS'], meta['LIKEDISTRO'])

print "analyzer starting up: imageId="+str(imgid) + " meta="+str(meta) + " distrodict="+str(distrodict)

if distrodict['flavor'] not in ['RHEL', 'DEB', 'BUSYB', 'ALPINE']:
    sys.exit(0)

pkgsall = {}
pkgfilesall = {}
pkgsplussource = {}

if distrodict['flavor'] == "RHEL":
    try:
        rpms = anchore.anchore_utils.rpm_get_all_packages(unpackdir)
        for pkg in rpms.keys():
            pkgsall[pkg] = rpms[pkg]['version'] + "-" + rpms[pkg]['release']
    except Exception as err:
        print "WARN: failed to generate RPM package list: " + str(err)

    try:
        rpmfiles = anchore.anchore_utils.rpm_get_all_pkgfiles(unpackdir)
        for pkgfile in rpmfiles.keys():
            pkgfilesall[pkgfile] = "RPMFILE"
    except Exception as err:
        print "WARN: failed to get file list from RPMs: " + str(err)

elif distrodict['flavor'] == "DEB":
    try:
        (all_packages, actual_packages, other_packages) = anchore.anchore_utils.dpkg_get_all_packages(unpackdir)
    
        for p in actual_packages.keys():
            pkgsall[p] = actual_packages[p]['version']

        for p in all_packages.keys():
            pkgsplussource[p] = all_packages[p]['version']

        if len(other_packages) > 0:
            for p in other_packages.keys():
                for v in other_packages[p]:
                    pkgsplussource[p] = v['version']
    except Exception as err:
        print "WARN: failed to get package list from DPKG: " + str(err)

    try:
        dpkgfiles = anchore.anchore_utils.dpkg_get_all_pkgfiles(unpackdir)
        for pkgfile in dpkgfiles.keys():
            pkgfilesall[pkgfile] = "DPKGFILE"

    except Exception as err:
        print "WARN: failed to get file list from DPKGs: " + str(err)

elif distrodict['flavor'] == 'ALPINE':
    try:
        apkgs = anchore.anchore_utils.apkg_get_all_pkgfiles(unpackdir)
        for pkg in apkgs.keys():
            # base
            if apkgs[pkg]['release'] != "N/A":
                pvers = apkgs[pkg]['version']+"-"+apkgs[pkg]['release']
                #pkgsall[pkg] = apkgs[pkg]['version']+"-"+apkgs[pkg]['release']
            else:
                pvers = apkgs[pkg]['version']
                #pkgsall[pkg] = apkgs[pkg]['version']
            pkgsall[pkg] = pvers
            pkgsplussource[pkg] = pvers

            # source package
            if 'sourcepkg' in apkgs[pkg] and apkgs[pkg]['sourcepkg']:
                spkg = apkgs[pkg]['sourcepkg']
                if spkg != pkg and spkg not in pkgsplussource:
                    pkgsplussource[spkg] = pvers

            # pkgfiles
            for pkgfile in apkgs[pkg]['files']:
                pkgfilesall[pkgfile] = 'APKFILE'

    except Exception as err:
        print "WARN: failed to generate APK package list: " + str(err)

elif distrodict['flavor'] == "BUSYB":
    pkgsall["BusyBox"] = distrodict['fullversion']
else:
    pkgsall["Unknown"] = "0"

if pkgsall:
    ofile = os.path.join(outputdir, 'pkgs.all')
    anchore.anchore_utils.write_kvfile_fromdict(ofile, pkgsall)
    #anchore.anchore_utils.save_analysis_output(imgid, 'package_list', 'pkgs.all', pkgsall)
if pkgfilesall:
    ofile = os.path.join(outputdir, 'pkgfiles.all')
    anchore.anchore_utils.write_kvfile_fromdict(ofile, pkgfilesall)
    #anchore.anchore_utils.save_analysis_output(imgid, 'package_list', 'pkgfiles.all', pkgfilesall)
if pkgsplussource:
    ofile = os.path.join(outputdir, 'pkgs_plus_source.all')
    anchore.anchore_utils.write_kvfile_fromdict(ofile, pkgsplussource)
    #anchore.anchore_utils.save_analysis_output(imgid, 'package_list', 'pkgs_plus_source.all', pkgsplussource)

sys.exit(0)



#!/usr/bin/env python

import sys
import os
import re
import traceback
import tarfile, io

import anchore.anchore_utils

def get_retrieved_file(imgid, srcfile, dstdir):
    ret = list()

    extractall = False
    if srcfile == 'all':
        extractall = True

    thedstdir = os.path.join(dstdir, imgid)

    tarfiles = list()
    namespaces = anchore.anchore_utils.load_files_namespaces(imgid)
    if namespaces:
        for namespace in namespaces:
            stored_data_tarfile = anchore.anchore_utils.load_files_tarfile(imgid, namespace)
            if stored_data_tarfile:
                tarfiles.append(stored_data_tarfile)
    else:
        stored_data_tarfile = anchore.anchore_utils.load_files_tarfile(imgid, 'retrieve_files')
        if stored_data_tarfile:
            tarfiles.append(stored_data_tarfile)

    for thetarfile in tarfiles:
        filetar = tarfile.open(thetarfile, mode='r:gz', format=tarfile.PAX_FORMAT)
        for ff in filetar.getmembers():
            patt = re.match("imageroot("+re.escape(srcfile)+")", ff.name)
            if extractall or patt:
                filetar.extract(ff, thedstdir)
                scrubbed_name = re.sub("imageroot", "", ff.name)
                ret.append([scrubbed_name, os.path.join(thedstdir, ff.name)])
        filetar.close()

    if namespaces:
        for namespace in namespaces:
            anchore.anchore_utils.del_files_cache(imgid)
    
    return(ret)

# main routine
try:
    config = anchore.anchore_utils.init_query_cmdline(sys.argv, "params: <source_filename> <destination_dir> ...\nhelp: Extract <source_filename> from stored files and copy to local host at <destination_dir>/<source_filename>.  Use 'all' as <source_filename> to extract all stored files into <destination_dir>")
except Exception as err:
    print str(err)
    sys.exit(1)

if not config:
    sys.exit(0)

if len(config['params']) < 2:
    print "Query requires input: <source_filename> <destination_dir> ..."
    sys.exit(1)

srcfile = config['params'][0]
dstdir = config['params'][1]

warns = list()
outlist = list()
outlist.append(["Image_Id", "Repo_Tags", "Stored_File_Name", "Output_Location"])

tags = "none"
if config['meta']['humanname']:
    tags = config['meta']['humanname']
imgid = config['meta']['shortId']

try:
    # handle the good case, something is found resulting in data matching the required columns
    retlist = get_retrieved_file(config['imgid'], srcfile, dstdir)
    if retlist:
        for ret in retlist:
            srcname = ret[0]
            dstname = ret[1]
            outlist.append([imgid, tags, srcname, dstname])
    else:
        warns.append("Could not find any stored files matching input '"+str(srcfile)+"' in image's stored files")

except Exception as err:
    # handle the case where something wrong happened
    warns.append("Unable to load stored files data - try re-analyzing image")
    import traceback
    traceback.print_exc()
    print str(err)

# handle the no match case
if len(outlist) < 1:
    #outlist.append(["NOMATCH", "NOMATCH", "NOMATCH"])
    pass

anchore.anchore_utils.write_kvfile_fromlist(config['output'], outlist)
if len(warns) > 0:
    anchore.anchore_utils.write_plainfile_fromlist(config['output_warns'], warns)

sys.exit(0)




#!/usr/bin/env python

import sys
import os
import re
import traceback
import anchore.anchore_utils

# main routine

try:
    config = anchore.anchore_utils.init_query_cmdline(sys.argv, "params: <STOP|WARN|GO> <STOP|WARN|GO> ...\nhelp: use 'all' to show all trigger/actions")
except:
    sys.exit(1)

if not config:
    sys.exit(0)

if 'all' in config['params']:
    config['params'] = ['STOP', 'WARN', 'GO']

outlist = list()
outlist.append(["Image_ID", "Repo_Tag", "Gate", "Trigger", "Action"])


report = anchore.anchore_utils.load_gates_eval_report(config['meta']['imageId'])
action = ""
for ge in report:
    try:
        action = ge['action']
        trigger = ge['trigger']
        gate = ge['check']
        if action == 'all' or action in config['params']:
            outlist.append([config['meta']['shortId'], config['meta']['humanname'], gate, trigger, action])
    except Exception as err:
        # bad record
        print "WARN: bad record detected: " + str(err)
        traceback.print_exc()
        
anchore.anchore_utils.write_kvfile_fromlist(config['output'], outlist)

sys.exit(0)



#!/usr/bin/env python

import sys
import os
import shutil
import re
import json
import time
import rpm
import subprocess

import anchore.anchore_utils

analyzer_name = "layer_info"

try:
    config = anchore.anchore_utils.init_analyzer_cmdline(sys.argv, analyzer_name)
except Exception as err:
    print str(err)
    sys.exit(1)

imgname = config['imgid']
outputdir = config['dirs']['outputdir']
unpackdir = config['dirs']['unpackdir']

#if not os.path.exists(outputdir):
#    os.makedirs(outputdir)

output = list()

try:
    hfile = os.path.join(unpackdir, "docker_history.json")
    if os.path.exists(hfile):
        with open(hfile, 'r') as FH:
            history = json.loads(FH.read())

        for record in history:
            clean_layer = re.sub("^sha256:", "", record['Id'])
            if clean_layer == '<missing>':
                clean_layer = "unknown"
                
            clean_createdBy = re.sub(r"^/bin/sh -c #\(nop\) ", "", record['CreatedBy'])
            line = {'layer':clean_layer, 'dockerfile_line':clean_createdBy, 'layer_sizebytes':str(record['Size'])}
            output.append(line)
    else:
        raise Exception("anchore failed to provide file '"+str(hfile)+"': cannot create layer info analyzer output")
except Exception as err:
    import traceback
    traceback.print_exc()
    raise err

ofile = os.path.join(outputdir, 'layers_to_dockerfile')
anchore.anchore_utils.write_kvfile_fromdict(ofile, {'dockerfile_to_layer_map':json.dumps(output)})

sys.exit(0)



#!/usr/bin/env python

import sys
import os
import json
import re
import shutil
import tarfile, io

import anchore.anchore_utils

def get_retrieved_file(imgid, srcfile, dstdir):
    ret = list()

    extractall = False
    if srcfile == 'all':
        extractall = True

    thedstdir = os.path.join(dstdir, imgid)

    tarfiles = list()
    namespaces = anchore.anchore_utils.load_files_namespaces(imgid)
    if namespaces:
        for namespace in namespaces:
            stored_data_tarfile = anchore.anchore_utils.load_files_tarfile(imgid, namespace)
            if stored_data_tarfile:
                tarfiles.append(stored_data_tarfile)
    else:
        stored_data_tarfile = anchore.anchore_utils.load_files_tarfile(imgid, 'retrieve_files')
        if stored_data_tarfile:
            tarfiles.append(stored_data_tarfile)

    for thetarfile in tarfiles:
        filetar = tarfile.open(thetarfile, mode='r:gz', format=tarfile.PAX_FORMAT)
        for ff in filetar.getmembers():
            patt = re.match("imageroot("+re.escape(srcfile)+")", ff.name)
            if extractall or patt:
                filetar.extract(ff, thedstdir)
                scrubbed_name = re.sub("imageroot", "", ff.name)
                ret.append([scrubbed_name, os.path.join(thedstdir, ff.name)])
        filetar.close()

    if namespaces:
        for namespace in namespaces:
            anchore.anchore_utils.del_files_cache(imgid)
    
    return(ret)

gate_name = "FILEPARSE_PASSWD"
triggers = {
    'FILENOTSTORED':
    {
        'description':'triggers if the /etc/passwd file is not present/stored in the evaluated image',
        'params':'none'
    },
    'USERNAMEMATCH':
    {
        'description':'triggers if specified username is found in the /etc/passwd file',
        'params':'USERNAMEBLACKLIST'
    },
    'USERIDMATCH':
    {
        'description':'triggers if specified user id is found in the /etc/passwd file',
        'params':'USERIDBLACKLIST'
    },
    'GROUPIDMATCH':
    {
        'description':'triggers if specified group id is found in the /etc/passwd file',
        'params':'GROUPIDBLACKLIST'
    },
    'SHELLMATCH':
    {
        'description':'triggers if specified login shell for any user is found in the /etc/passwd file',
        'params':'SHELLBLACKLIST'
    },
    'PENTRYMATCH':
    {
        'description':'triggers if specified entire passwd entry is found in the /etc/passwd file',
        'params':'PENTRYBLACKLIST'
    }
}

try:
    config = anchore.anchore_utils.init_gate_cmdline(sys.argv, gate_name, gate_help=triggers)
except Exception as err:
    print str(err)
    sys.exit(1)

if not config:
    print "ERROR: could not set up environment for gate"
    sys.exit(1)

if len(config['params']) <= 0:
    sys.exit(0)

imgid = config['imgid']
#outputdir = config['dirs']['outputdir']

try:
    params = config['params']
except:
    params = None

# do somthing
outlist = list()

# first, attempt to extract /etc/passwd
dstdir = anchore.anchore_utils.make_anchoretmpdir("/tmp")

retlist = list()
try:
    retlist = get_retrieved_file(config['imgid'], '/etc/passwd', dstdir)
except Exception as err:
    print "Exception: " + str(err)

if not retlist:
    outlist.append("FILENOTSTORED Cannot locate /etc/passwd in image's stored files archive: check analyzer settings")
else:
    users = {}
    with open(retlist[0][1], 'r') as FH:
        for l in FH.readlines():
            l = l.strip()
            try:
                pentry = l.split(":")
                users[pentry[0]] = pentry[1:]
            except:
                pass
    
    for pstr in params:
        try:
            (pkey, pvallist) = pstr.split("=")
            if pkey == 'USERNAMEBLACKLIST':
                for pval in pvallist.split(","):
                    try:
                        if pval in users:
                            pentry = str(':'.join([pval] + users[u]))
                            outlist.append("USERNAMEMATCH Blacklisted user '"+pval+"' found in image's /etc/passwd: pentry="+pentry)
                    except:
                        pass
            elif pkey == 'USERIDBLACKLIST':
                for pval in pvallist.split(","):
                    try:
                        for u in users.keys():
                            uid = users[u][1]
                            if str(uid) == pval:
                                pentry = str(':'.join([u] + users[u]))
                                outlist.append("USERIDMATCH Blacklisted uid '"+pval+"' found in image's /etc/passwd: pentry="+pentry)
                    except:
                        pass
            elif pkey == 'GROUPIDBLACKLIST':
                for pval in pvallist.split(","):
                    try:
                        for u in users.keys():
                            gid = users[u][2]
                            if str(gid) == pval:
                                pentry = str(':'.join([u] + users[u]))
                                outlist.append("GROUPIDMATCH Blacklisted gid '"+pval+"' found in image's /etc/passwd: pentry="+pentry)
                    except:
                        pass
            elif pkey == 'SHELLBLACKLIST':
                for pval in pvallist.split(","):
                    try:
                        for u in users.keys():
                            shellstr = users[u][5]
                            if str(shellstr) == pval:
                                pentry = str(':'.join([u] + users[u]))
                                outlist.append("SHELLMATCH Blacklisted shell '"+pval+"' found in image's /etc/passwd: pentry="+pentry)
                    except:
                        pass
            elif pkey == 'PENTRYBLACKLIST':
                for pval in pvallist.split(","):
                    try:
                        for u in users.keys():
                            pentry = str(':'.join([u] + users[u]))
                            if pval == pentry:
                                outlist.append("PENTRYMATCH Blacklisted pentry '"+pval+"' found in image's /etc/passwd: pentry="+pentry)
                    except:
                        pass
        except:
            pass

try:
    if dstdir:
        shutil.rmtree(dstdir)
except:
    pass

# write output
anchore.anchore_utils.save_gate_output(imgid, gate_name, outlist)

sys.exit(0)



import sys
import click

from anchore.cli.common import build_image_list, anchore_print, anchore_print_err, extended_help_option
from anchore import navigator, anchore_utils
from anchore.util import contexts

config = {}
imagelist = []
nav = None

@click.group(short_help='Commands to generate/review audit reports')
@click.option('--image', help='Process specified image ID', metavar='<imageid>')
@click.option('--imagefile', help='Process image IDs listed in specified file', type=click.Path(exists=True), metavar='<file>')
@click.option('--include-allanchore', help='Include all images known by anchore', is_flag=True)
@click.pass_context
@click.pass_obj
@extended_help_option()
def audit(anchore_config, ctx, image, imagefile, include_allanchore):
    """
    Image IDs can be specified as hash ids, repo names (e.g. centos), or tags (e.g. centos:latest).
    """

    global config, imagelist, nav
    ecode = 0
    success = True
    config = anchore_config

    #include_allanchore = True

    if image and imagefile:
        raise click.BadOptionUsage('Can only use one of --image, --imagefile')

    #if image or imagefile:
    #    include_allanchore = False

    try:
        imagedict = build_image_list(anchore_config, image, imagefile, not (image or imagefile), include_allanchore)
        imagelist = imagedict.keys()
        try:
            ret = anchore_utils.discover_imageIds(imagelist)
        except ValueError as err:
            raise err
        else:
            imagelist = ret

    except Exception as err:
        anchore_print_err("could not load input images")
        sys.exit(1)

def init_nav_contexts():
    try:
        # use the obj from the current click context. This is a bit hacky, but works as long as this method is
        # invoked in an execution context of click
        anchore_config = click.get_current_context().obj
        nav = navigator.Navigator(anchore_config=anchore_config, imagelist=imagelist, allimages=contexts['anchore_allimages'])
        return nav
    except Exception as err:
        anchore_print_err("explore operation failed")
        success = False
        ecode = 1

    if not success:
        contexts['anchore_allimages'].clear()
        sys.exit(ecode)

@audit.command(short_help='Generate summarized report of specified images.')
#@click.option('--all', help='Show all images including intermediate layer images (default shows only images with a type)', is_flag=True)
#@click.option('--details', help='Show many details in the report (including full image details if --json is supplied)', is_flag=True)
@extended_help_option()
#def report(all, details):
def report():
    """
    Show analysis report of the specified image(s).

    The analysis report includes information on:

    \b
    Image Id - The image id (as a hash)

    Type - The type of image (--imagetype option used when anchore analyze was run)

    CurrentTags - The current set of repo tags on the image

    AllTags - The set of all repo tags that have been on the image during analysis passes

    GateStatus - The overall aggregate gate output status: GO|STOP|WARN

    Size - The size in bytes of the image on disk
    
    Counts - The counts for various attributes of the images such as packages, files, and suid files

    BaseDiffs - Differences of this image from its base image

    Report outputs these entries in a table format by default.
    """
    ecode = 0

    try:
        nav = init_nav_contexts()
        result = nav.generate_reports()

        #result = generate_reports(imagelist, showall=all, showdetails=details)

        if result:
            anchore_utils.print_result(config, result)

    except:
        anchore_print_err("operation failed")
        ecode = 1

    contexts['anchore_allimages'].clear()
    sys.exit(ecode)


def generate_reports(imagelist, showall=True, showdetails=True):
    ret = {}

    if showdetails:
        header = ['Image_Id', '*Type', 'Current_Tags', 'All_Tags', 'Is_Analyzed', 'Gate_Status', 'Size(bytes)', 'Counts', 'Base_Diffs']
    else:
        header = ['Image_Id', '*Type', 'Tags', 'Is_Analyzed', 'Gate_Status', 'Size(bytes)']

    for imageId in imagelist:
        isanalyzed = str(anchore_utils.is_image_analyzed(imageId))        
        ireport = anchore_utils.load_image_report(imageId)
        if ireport:
            usertype = str(ireport['meta']['usertype'])
            currtags = ','.join(ireport['anchore_current_tags'])
            alltags = ','.join(ireport['anchore_all_tags'])
        else:
            usertype = "None"
            try:
                idocker = contexts['docker_images'][imageId]
                currtags = ','.join(idocker['RepoTags'])
                alltags = currtags
            except:
                currtags = alltags = "N/A"

        if not showall and ((not usertype or usertype.lower() == 'none') and (not currtags and not alltags)):
            continue

        if ireport:
            baseId = str(ireport['familytree'][0])
            sizebytes = str(ireport['meta']['sizebytes'])
            shortId = str(ireport['meta']['shortId'])

        else:
            baseId = "N/A"
            sizebytes = "N/A"
            shortId = imageId[0:12]

        gates_eval_report = anchore_utils.load_gates_eval_report(imageId)
        record = {
            'image_report': ireport,
            'analysis_report': {},
            'gates_report': {},
            'gates_eval_report': gates_eval_report,
            'result': {
                'header':header,
                'rows': list()
            }
        }

        if showdetails: 
            record['analysis_report'] = anchore_utils.load_analysis_report(imageId)
            record['gates_report'] = anchore_utils.load_gates_report(imageId)

        
        gateaction = 'UNKNOWN'
        for g in gates_eval_report:
            if g['trigger'] == 'FINAL':
                gateaction = g['action']
                break

        if showdetails:
            try:
                pnum = str(len(anchore_utils.load_analysis_output(imageId, 'package_list', 'pkgs.all').keys()))
            except:
                pnum = "N/A"
            try:
                fnum = str(len(anchore_utils.load_analysis_output(imageId, 'file_list', 'files.all').keys()))
            except:
                fnum = "N/A"
            try:
                snum = str(len(anchore_utils.load_analysis_output(imageId, 'file_suids', 'files.suids').keys()))
            except:
                snum = "N/A"

            analysis_str = ' '.join(["PKGS="+pnum, "FILES="+fnum, "SUIDFILES="+snum])

            compare_str = "N/A"

            if imageId != baseId:
                diffdata = anchore_utils.diff_images(imageId, baseId)            
                record['base_compare_data'] = diffdata
                pnum = "N/A"
                if 'package_list' in diffdata and 'pkgs.all' in diffdata['package_list']:
                    for module_type in diffdata['package_list']['pkgs.all']:
                        pnum = str(len(diffdata['package_list']['pkgs.all'][module_type]))
                        break

                fnum = "N/A"
                if 'file_list' in diffdata and 'files.all' in diffdata['file_list']:
                    for module_type in diffdata['file_list']['files.all']:
                        fnum = str(len(diffdata['file_list']['files.all'][module_type]))

                snum = "N/A"
                if 'file_suids' in diffdata and 'files.suids' in diffdata['file_suids']:
                    for module_type in diffdata['file_suids']['files.suids']:
                        snum = str(len(diffdata['file_suids']['files.suids'][module_type]))

                compare_str = ' '.join(["PKGS="+pnum, "FILES="+fnum, "SUIDFILES="+snum])

            row = [ shortId, usertype, currtags, alltags, isanalyzed, gateaction, sizebytes, analysis_str, compare_str ]
        else:
            row = [ shortId, usertype, currtags, isanalyzed, gateaction, sizebytes]

        record['result']['rows'].append(row)

        ret[imageId] = record

        import json
        with open("/tmp/j", 'w') as OFH:
            OFH.write(json.dumps(ret))

    return ret



import json
import re
import os
import shutil
import time
import logging
import tarfile

from anchore import anchore_utils

DEVNULL = open(os.devnull, 'wb')

class AnchoreImageDB(object):
    _logger = logging.getLogger(__name__)

    def __init__(self, config={}):
        self.config = config
        self.initialized = False
        self.version = None
        self.anchore_version_string = None
        self.anchore_db_version_string = None
        try:
            from anchore import version as anchore_version_string
            self.anchore_version_string = anchore_version_string
            self.anchore_db_version_string = self.anchore_version_string
            try:
                patt = re.match("([0-9]+)\.([0-9]+)\.(.*)", self.anchore_version_string)
                if patt:
                    (major, minor, subminor) = patt.group(1,2,3)
                    self.anchore_db_version_string = '.'.join([major, minor])
            except Exception as err:
                pass
                
            self.version = {'anchore_version': self.anchore_version_string, 'db_version': self.anchore_db_version_string}
        except ValueError as err:
            raise err
        except Exception as err:
            raise err

        #self.initialized = True        

    def __del__(self):
        if self.initialized:
            pass

    def get_version(self):
        return(self.version)

    def check(self):
        return(self.initialized)

    def is_image_present(self, imageId, imagelist=None):
        self._logger.debug("AnchoreDB: parent unimplemented")
        return(False)

    def is_image_analyzed(self, imageId):
        self._logger.debug("AnchoreDB: parent unimplemented: driver has not implemented this anchoreDB operation")
        return(False)

    def get_image_list(self):
        self._logger.debug("AnchoreDB: parent unimplemented: driver has not implemented this anchoreDB operation")
        return(False)

    def load_all_images(self):
        self._logger.debug("AnchoreDB: parent unimplemented: driver has not implemented this anchoreDB operation")
        return(False)

    def load_all_images_iter(self):
        self._logger.debug("AnchoreDB: parent unimplemented: driver has not implemented this anchoreDB operation")
        return(False)

    def delete_image(self, imageId):
        self._logger.debug("AnchoreDB: parent unimplemented: driver has not implemented this anchoreDB operation")
        return(False)

    def load_image(self, imageId):
        self._logger.debug("AnchoreDB: parent unimplemented: driver has not implemented this anchoreDB operation")
        return(False)

    def load_image_new(self, imageId):
        self._logger.debug("AnchoreDB: parent unimplemented: driver has not implemented this anchoreDB operation")
        return(False)

    def create_image(self, imageId):
        self._logger.debug("AnchoreDB: parent unimplemented: driver has not implemented this anchoreDB operation")
        return(False)

    def make_image_structure(self, imageId):
        self._logger.debug("AnchoreDB: parent unimplemented: driver has not implemented this anchoreDB operation")
        return(False)

    def save_image_new(self, imageId, report=None):
        self._logger.debug("AnchoreDB: parent unimplemented: driver has not implemented this anchoreDB operation")
        return(False)

    def load_analysis_report(self, imageId):
        self._logger.debug("AnchoreDB: parent unimplemented: driver has not implemented this anchoreDB operation")
        return(False)

    def save_analysis_report(self, imageId, report):
        self._logger.debug("AnchoreDB: parent unimplemented: driver has not implemented this anchoreDB operation")
        return(False)

    def list_analysis_outputs(self, imageId):
        self._logger.debug("AnchoreDB: parent unimplemented: driver has not implemented this anchoreDB operation")
        return(False)

    def load_analyzer_manifest(self, imageId):
        self._logger.debug("AnchoreDB: parent unimplemented: driver has not implemented this anchoreDB operation")
        return(False)

    def save_analyzer_manifest(self, imageId, data):
        self._logger.debug("AnchoreDB: parent unimplemented: driver has not implemented this anchoreDB operation")
        return(False)

    def load_analysis_output(self, imageId, module_name, module_value, module_type=None):
        self._logger.debug("AnchoreDB: parent unimplemented: driver has not implemented this anchoreDB operation")
        return(False)

    def save_analysis_output(self, imageId, module_name, module_value, data, module_type=None, directory_data=False):
        self._logger.debug("AnchoreDB: parent unimplemented: driver has not implemented this anchoreDB operation")
        return(False)

    def load_gates_report(self, imageId):
        self._logger.debug("AnchoreDB: parent unimplemented: driver has not implemented this anchoreDB operation")
        return(False)

    def save_gates_report(self, imageId, report):
        self._logger.debug("AnchoreDB: parent unimplemented: driver has not implemented this anchoreDB operation")
        return(False)

    def load_gate_output(self, imageId, gate_name):
        self._logger.debug("AnchoreDB: parent unimplemented: driver has not implemented this anchoreDB operation")
        return(False)

    def list_gate_outputs(self, imageId):
        self._logger.debug("AnchoreDB: parent unimplemented: driver has not implemented this anchoreDB operation")
        return(False)

    def save_gate_output(self, imageId, gate_name, data):
        self._logger.debug("AnchoreDB: parent unimplemented: driver has not implemented this anchoreDB operation")
        return(False)

    def save_gate_help_output(self, gate_help):
        self._logger.debug("AnchoreDB: parent unimplemented: driver has not implemented this anchoreDB operation")
        return(False)

    def save_gate_eval_output(self, imageId, gate_name, data):
        self._logger.debug("AnchoreDB: parent unimplemented: driver has not implemented this anchoreDB operation")
        return(False)

    def del_gate_eval_output(self, imageId, gate_name):
        self._logger.debug("AnchoreDB: parent unimplemented: driver has not implemented this anchoreDB operation")
        return(False)

    def load_gates_eval_report(self, imageId):
        self._logger.debug("AnchoreDB: parent unimplemented: driver has not implemented this anchoreDB operation")
        return(False)

    def save_gates_eval_report(self, imageId, report):
        self._logger.debug("AnchoreDB: parent unimplemented: driver has not implemented this anchoreDB operation")
        return(False)

    def load_gate_policy(self, imageId):
        self._logger.debug("AnchoreDB: parent unimplemented: driver has not implemented this anchoreDB operation")
        return(False)

    def save_gate_policy(self, imageId, data):
        self._logger.debug("AnchoreDB: parent unimplemented: driver has not implemented this anchoreDB operation")
        return(False)

    def del_gate_policy(self, imageId):
        self._logger.debug("AnchoreDB: parent unimplemented: driver has not implemented this anchoreDB operation")
        return(False)

    def load_gate_whitelist(self, imageId):
        self._logger.debug("AnchoreDB: parent unimplemented: driver has not implemented this anchoreDB operation")
        return(False)

    def save_gate_whitelist(self, imageId, data):
        self._logger.debug("AnchoreDB: parent unimplemented: driver has not implemented this anchoreDB operation")
        return(False)

    def load_image_report(self, imageId):
        self._logger.debug("AnchoreDB: parent unimplemented: driver has not implemented this anchoreDB operation")
        return(False)

    def save_image_report(self, imageId, report):
        self._logger.debug("AnchoreDB: parent unimplemented: driver has not implemented this anchoreDB operation")
        return(False)

    def save_files(self, imageId, namespace, rootfsdir, files):
        self._logger.debug("AnchoreDB: parent unimplemented: driver has not implemented this anchoreDB operation")
        return(False)

    def load_files_tarfile(self, imageId, namespace):
        self._logger.debug("AnchoreDB: parent unimplemented: driver has not implemented this anchoreDB operation")
        return(False)

    def save_files_tarfile(self, imageId, namespace, tarfile):
        self._logger.debug("AnchoreDB: parent unimplemented: driver has not implemented this anchoreDB operation")
        return(False)

    def load_files_namespaces(self, imageId):
        self._logger.debug("AnchoreDB: parent unimplemented: driver has not implemented this anchoreDB operation")
        return(False)

    def load_files_metadata(self, imageId, namespace):
        self._logger.debug("AnchoreDB: parent unimplemented: driver has not implemented this anchoreDB operation")
        return(False)

    def del_files_cache(self, imageId, namespace):
        self._logger.debug("AnchoreDB: parent unimplemented: driver has not implemented this anchoreDB operation")
        return(False)

    def load_feedmeta(self):
        self._logger.debug("AnchoreDB: parent unimplemented: driver has not implemented this anchoreDB operation")
        return(False)

    def save_feedmeta(self, feedmeta):
        self._logger.debug("AnchoreDB: parent unimplemented: driver has not implemented this anchoreDB operation")
        return(False)

    def create_feed(self, feed):
        self._logger.debug("AnchoreDB: parent unimplemented: driver has not implemented this anchoreDB operation")
        return(False)

    def create_feedgroup(self, feed, group):
        self._logger.debug("AnchoreDB: parent unimplemented: driver has not implemented this anchoreDB operation")
        return(False)

    def delete_feed(self, feed):
        self._logger.debug("AnchoreDB: parent unimplemented: driver has not implemented this anchoreDB operation")
        return(False)

    def save_feed_group_data(self, feed, group, datafilename, data):
        self._logger.debug("AnchoreDB: parent unimplemented: driver has not implemented this anchoreDB operation")
        return(False)

    def load_feed_group_data(self, feed, group, datafilename):
        self._logger.debug("AnchoreDB: parent unimplemented: driver has not implemented this anchoreDB operation")
        return(False)



#!/usr/bin/env python

import sys
import os
import re

import anchore.anchore_image
import anchore.anchore_utils

# main routine

try:
    config = anchore.anchore_utils.init_query_cmdline(sys.argv, "params: <image ID to compare to> <another image ID> exclude=</some/path> exclude=</some/other/path> ...\nhelp: use 'base' as an imageId for comparison against base image, and exclude=</some/path> to exclude results that begin with the supplied path string")
except:
    sys.exit(1)

if not config:
    sys.exit(0)

if len(config['params']) <= 0:
    config['params'] = ['base']

fids = list()
excludes = list()
for p in config['params']:
    try:
        (key, value) = p.split('=')
        if key == 'exclude':
            excludes.append(value)
    except:
        fids.append(p)

outlist = list()
warns = list()
outlist.append(["Image_Id", "Repo_Tag", "Compare_Image_Id", "File", "Input_Image_File_Checksum","Compare_Image_Checksum"])

allimages = {}
imageId = config['imgid']

try:
    image = anchore.anchore_image.AnchoreImage(imageId, config['anchore_config']['image_data_store'], allimages)
    ipkgs = image.get_allfiles()

    for fid in fids:
        try:
            fimageId = False
            if fid == 'base':
                fimageId = image.get_earliest_base()
            else:
                try:
                    fimageId = anchore.anchore_utils.discover_imageId(fid)
                except ValueError as err:
                    warns.append("Cannot lookup imageId specified as input parameter: " + fid)

            if fimageId:
                if not anchore.anchore_utils.is_image_analyzed(fimageId):
                    raise Exception("imageId ("+str(fimageId)+") is not analyzed or analysis failed")

                fimage = anchore.anchore_image.AnchoreImage(fimageId, config['anchore_config']['image_data_store'], allimages)
                image_report = anchore.anchore_utils.diff_images(image.meta['imageId'], fimage.meta['imageId'])
                fpkgs = fimage.get_allfiles()

                # do some checks
                if not image_report or 'file_checksums' not in image_report:
                    raise Exception("could not load file_checksums data after diff_images")

                csumkey = 'files.md5sums'
                if 'files.sha256sums' in image_report['file_checksums']:
                    csumkey = 'files.sha256sums'

                for module_type in ['base', 'extra', 'user']:
                    if module_type in image_report['file_checksums'][csumkey]:
                        for pkg in image_report['file_checksums'][csumkey][module_type].keys():

                            skip = False
                            for e in excludes:
                                if re.match("^"+e, pkg):
                                    skip = True
                                    break

                            if skip:
                                continue

                            status = image_report['file_checksums'][csumkey][module_type][pkg]
                            ivers = ipkgs.pop(pkg, "N/A")
                            pvers = fpkgs.pop(pkg, "N/A")

                            if status == 'VERSION_DIFF':
                                outlist.append([config['meta']['shortId'], config['meta']['humanname'], fimage.meta['shortId'], pkg, ivers, pvers])
                            elif status == 'INIMG_NOTINBASE':
                                outlist.append([config['meta']['shortId'], config['meta']['humanname'], fimage.meta['shortId'], pkg, ivers, "NOTINSTALLED"])
                            elif status == 'INBASE_NOTINIMG':
                                outlist.append([config['meta']['shortId'], config['meta']['humanname'], fimage.meta['shortId'], pkg, "NOTINSTALLED", pvers])

        except Exception as err:
            import traceback
            traceback.print_exc()
            warns.append("problem comparing image ("+str(imageId)+") with image ("+str(fid)+"): exception: " + str(err))
except Exception as err:
    warns.append("failed to run query: exception: " + str(err))

anchore.anchore_utils.write_kvfile_fromlist(config['output'], outlist)
if len(warns) > 0:
    anchore.anchore_utils.write_plainfile_fromlist(config['output_warns'], warns)

allimages.clear()    
sys.exit(0)



import sys
import os
import re
import json
import getpass
import click
import yaml
import time
import shutil

from anchore.cli.common import anchore_print, anchore_print_err
from anchore.util import contexts
from anchore import anchore_utils, anchore_auth, anchore_feeds

config = {}

@click.group(name='system', short_help='System level operations.')
@click.pass_obj
def system(anchore_config):
    global config
    config = anchore_config

@system.command(name='status', short_help="Show system status.")
@click.option('--conf', is_flag=True, help='Output the currently used configuration yaml content')
def status(conf):
    """
    Show anchore system status.
    """

    ecode = 0
    try:
        if conf:
            if config.cliargs['json']:
                anchore_print(config.data, do_formatting=True)
            else:
                anchore_print(yaml.safe_dump(config.data, indent=True, default_flow_style=False))
        else:
            result = {}
            if contexts['anchore_db'].check():
                result["anchore_db"] = "OK"
            else:
                result["anchore_db"] = "NOTINITIALIZED"

            if anchore_feeds.check():
                result["anchore_feeds"] = "OK"
            else:
                result["anchore_feeds"] = "NOTSYNCED"

            afailed = False
            latest = 0
            for imageId in contexts['anchore_db'].load_all_images().keys():
                amanifest = anchore_utils.load_analyzer_manifest(imageId)
                for module_name in amanifest.keys():
                    try:
                        if amanifest[module_name]['timestamp'] > latest:
                            latest = amanifest[module_name]['timestamp']
                        if amanifest[module_name]['status'] != 'SUCCESS':
                            analyzer_failed_imageId = imageId
                            analyzer_failed_name = module_name
                            afailed = True
                    except:
                        pass

            if latest == 0:
                result["analyzer_status"] = "NODATA"
            elif afailed:
                result["analyzer_status"] = "FAIL ("+analyzer_failed_imageId+")"
                result["analyzer_latest_run"] = time.ctime(latest)
            else:
                result["analyzer_status"] = "OK"
                result["analyzer_latest_run"] = time.ctime(latest)
   
            anchore_print(result, do_formatting=True)

    except Exception as err:
        anchore_print_err('operation failed')
        ecode = 1

    sys.exit(ecode)

@system.command(name='backup', short_help="Backup an anchore installation to a tarfile.")
@click.argument('outputdir', type=click.Path())
def backup(outputdir):
    """
    Backup an anchore installation to a tarfile.
    """

    ecode = 0
    try:
        anchore_print('Backing up anchore system to directory '+str(outputdir)+' ...')
        backupfile = config.backup(outputdir)
        anchore_print({"anchore_backup_tarball":str(backupfile)}, do_formatting=True)
    except Exception as err:
        anchore_print_err('operation failed')
        ecode = 1

    sys.exit(ecode)

@system.command(name='restore', short_help="Restore an anchore installation from a previously backed up tar file.")
@click.argument('inputfile', type=click.File('rb'))
@click.argument('destination_root', type=click.Path(), default='/')
def restore(inputfile, destination_root):
    """
    Restore an anchore installation from a previously backed up tar file.
    """

    ecode = 0
    try:
        anchore_print('Restoring anchore system from backup file %s ...' % (str(inputfile.name)))
        restoredir = config.restore(destination_root, inputfile)
        anchore_print("Anchore restored.")
    except Exception as err:
        anchore_print_err('operation failed')
        ecode = 1

    sys.exit(ecode)

@system.command(name='exportdb')
@click.option('--outdir', help='output directory for exported anchore DB', required=True, metavar='<export directory>')
def exportdb(outdir):
    """Export all anchore images to JSON files"""
    ecode = 0
    try:
        imgdir = os.path.join(outdir, "images")
        feeddir = os.path.join(outdir, "feeds")
        storedir = os.path.join(outdir, "storedfiles")

        for d in [outdir, imgdir, feeddir, storedir]:
            if not os.path.exists(d):
                os.makedirs(d)

        anchore_print("exporting images...")
        imagelist = anchore_utils.get_image_list().keys()
        for imageId in imagelist:
            thefile = os.path.join(imgdir, imageId+".json")
            if not os.path.exists(thefile):
                with open(thefile, 'w') as OFH:
                    OFH.write(json.dumps(contexts['anchore_db'].load_image_new(imageId)))

            stored_namespaces = contexts['anchore_db'].load_files_namespaces(imageId)
            for namespace in stored_namespaces:
                stored_files = contexts['anchore_db'].load_files_tarfile(imageId, namespace)
                if os.path.exists(stored_files):
                    thedir = os.path.join(storedir, imageId, namespace)
                    if not os.path.exists(thedir):
                        os.makedirs(thedir)
                    thefile = os.path.join(thedir, "stored_files.tar.gz")
                    shutil.copy(stored_files, thefile)

        anchore_print("exporting feeds...")
        feedmeta = contexts['anchore_db'].load_feedmeta()
        thefile = os.path.join(feeddir, "feedmeta.json")
        with open(thefile, 'w') as OFH:
            OFH.write(json.dumps(feedmeta))

        for feed in feedmeta:
            feedobj = feedmeta[feed]
            for group in feedobj['groups']:
                groupobj = feedobj['groups'][group]
                datafiles = groupobj.pop('datafiles', [])
                for datafile in datafiles:
                    thedir = os.path.join(feeddir, feed, group)
                    if not os.path.exists(thedir):
                        os.makedirs(thedir)
                    thefile = os.path.join(thedir, datafile)
                    if not os.path.exists(thefile):
                        with open(thefile, 'w') as OFH:
                            OFH.write(json.dumps(contexts['anchore_db'].load_feed_group_data(feed, group, datafile)))

    except Exception as err:
        anchore_print_err("operation failed: " + str(err))
        ecode = 1

    sys.exit(ecode)

@system.command(name='importdb')
@click.option('--indir', help='directory from previously exported anchore DB', required=True, metavar='<export directory>')
def importdb(indir):
    """Import a previously exported anchore DB"""
    ecode = 0
    try:
        imgdir = os.path.join(indir, "images")
        feeddir = os.path.join(indir, "feeds")
        storedir = os.path.join(indir, "storedfiles")

        for d in [indir, imgdir, feeddir, storedir]:
            if not os.path.exists(d):
                raise Exception ("specified directory "+str(indir)+" does not appear to be complete (missing "+str(d)+")")

                
        anchore_print("importing images...")
        #imagelist = []
        for ifile in os.listdir(imgdir):
            patt = re.match("(.*)\.json", ifile)
            if patt:
                imageId = patt.group(1)

                if contexts['anchore_db'].is_image_present(imageId):
                    anchore_print("\timage ("+str(imageId)+") already exists in DB, skipping import.")
                else:
                    #imagelist.append(patt.group(1))
                    thefile = os.path.join(imgdir, ifile)
                    with open(thefile, 'r') as FH:
                        imagedata = json.loads(FH.read())
                    try:
                        rc = contexts['anchore_db'].save_image_new(imageId, report=imagedata)
                        if not rc:
                            contexts['anchore_db'].delete_image(imageId)
                            raise Exception("save to anchore DB failed")
                    except Exception as err:
                        contexts['anchore_db'].delete_image(imageId)
                        raise err

                    thedir = os.path.join(storedir, imageId)
                    if os.path.exists(thedir):
                        for namespace in os.listdir(thedir):
                            thefile = os.path.join(thedir, namespace, "stored_files.tar.gz")
                            if os.path.exists(thefile):
                                contexts['anchore_db'].save_files_tarfile(imageId, namespace, thefile)

                    anchore_print("\timage ("+str(imageId)+") imported.")

        anchore_print("importing feeds...")
        thefile = os.path.join(feeddir, "feedmeta.json")
        with open(thefile, 'r') as FH:
            feedmeta = json.loads(FH.read())

        if feedmeta:
            contexts['anchore_db'].save_feedmeta(feedmeta)

        for feed in feedmeta:
            feedobj = feedmeta[feed]
            for group in feedobj['groups']:
                groupobj = feedobj['groups'][group]
                datafiles = groupobj.pop('datafiles', [])
                for datafile in datafiles:
                    thedir = os.path.join(feeddir, feed, group)
                    thefile = os.path.join(thedir, datafile)
                    if not os.path.exists(thefile):
                        pass
                    else:
                        with open(thefile, 'r') as FH:
                            contexts['anchore_db'].save_feed_group_data(feed, group, datafile, json.loads(FH.read()))
                    anchore_print("\tfeed ("+feed+" " + group + " " + datafile + ") imported")

        #TODO import stored files

    except Exception as err:
        anchore_print_err("operation failed: " + str(err))
        ecode = 1

    sys.exit(ecode)




#!/usr/bin/env python

import sys
import os
import stat
import re
import json
import traceback

import anchore.anchore_utils

# main routine

try:
    config = anchore.anchore_utils.init_query_cmdline(sys.argv, "params: <package name> <package name> ...\nhelp: use 'all' to show all packages")
except Exception as err:
    print str(err)
    sys.exit(1)

if not config:
    sys.exit(0)

if len(config['params']) <= 0:
    print "Query requires input: <package name> <package name> ..."

outlist = list()
warns = list()
outlist.append(["Image_Id", "Repo_Tags", "Package", "Version"])

try:
    # handle the good case, something is found resulting in data matching the required columns

    pkg_data = anchore.anchore_utils.load_analysis_output(config['imgid'], 'package_list', 'pkgs.all')
    for pname in pkg_data.keys():
        match = False
        if 'all' in config['params']:
            match = True
        else:
            for prefix in config['params']:
                if re.match(prefix, pname):
                    match = True
                    break
        
        if not match:
            continue

        outlist.append([config['meta']['shortId'], config['meta']['humanname'], pname, pkg_data[pname]])

except Exception as err:
    # handle the case where something wrong happened
    import traceback
    traceback.print_exc()
    warns.append("query threw an exception: " + str(err))

anchore.anchore_utils.write_kvfile_fromlist(config['output'], outlist)
if len(warns) > 0:
    anchore.anchore_utils.write_plainfile_fromlist(config['output_warns'], warns)

sys.exit(0)



import json
import yaml
import time

import os
import shutil
import sys
import re
import rpm
import subprocess
import docker
import io
import tarfile
import urllib
import hashlib
import random
import traceback

from stat import *
from prettytable import PrettyTable
from textwrap import fill
from rpmUtils.miscutils import splitFilename

import logging

import anchore_image, anchore_image_db
from configuration import AnchoreConfiguration
from anchore.util import contexts, scripting
import anchore_auth
import anchore_feeds

_logger = logging.getLogger(__name__)

def init_analyzer_cmdline(argv, name):
    ret = {}

    if len(argv) < 4:
        print "ERROR: invalid input"
        raise Exception

    anchore_conf = AnchoreConfiguration()
    anchore_common_context_setup(anchore_conf)


    ret['analyzer_config'] = None
    anchore_analyzer_configfile = '/'.join([anchore_conf.config_dir, 'analyzer_config.yaml'])
    if os.path.exists(anchore_analyzer_configfile):
        try:
            with open(anchore_analyzer_configfile, 'r') as FH:
                anchore_analyzer_config = yaml.safe_load(FH.read())
        except Exception as err:
            print "ERROR: could not parse the analyzer_config.yaml - exception: " + str(err)
            raise err

        if anchore_analyzer_config and name in anchore_analyzer_config:
            ret['analyzer_config'] = anchore_analyzer_config[name]

    ret['anchore_config'] = anchore_conf.data

    ret['name'] = name

    FH=open(argv[0], 'r')
    ret['selfcsum'] = hashlib.md5(FH.read()).hexdigest()
    FH.close()
    ret['imgid'] = argv[1]

    fullid = discover_imageId(argv[1])
    if fullid:
        ret['imgid_full'] = fullid
    else:
        ret['imgid_full'] = ret['imgid']

    ret['dirs'] = {}
    ret['dirs']['datadir'] = argv[2]
    ret['dirs']['outputdir'] = '/'.join([argv[3], "analyzer_output", name])
    ret['dirs']['unpackdir'] = argv[4]

    for d in ret['dirs'].keys():
        if not os.path.isdir(ret['dirs'][d]):
            try:
                os.makedirs(ret['dirs'][d])
            except Exception as err:
                print "ERROR: cannot find/create input dir '"+ret['dirs'][d]+"'"
                raise err

    return(ret)

def init_gate_cmdline(argv, gate_name, gate_help={}):
    if len(argv) > 2 and argv[2] == 'anchore_get_help':
        if gate_help:
            thefile = os.path.join(argv[1], gate_name + ".help")
            update_file_jsonstr(json.dumps(gate_help), thefile)
        sys.exit(0)
        
    ret = init_query_cmdline(argv, gate_name)
    return(ret)

def init_query_cmdline(argv, paramhelp):
    ret = {}

    logging.basicConfig(format='%(asctime)-15s %(levelname)s %(filename)s:%(funcName)s %(message)s', level='INFO')
    
    if len(argv) == 2 and re.match(".*help.*", argv[1]):
        print paramhelp
        return (False)

    if len(argv) < 5:
        print "ERROR: invalid input"
        raise Exception

    anchore_conf = AnchoreConfiguration()
    anchore_common_context_setup(anchore_conf)

    ret['anchore_config'] = anchore_conf.data

    ret['name'] = argv[0].split('/')[-1]
    ret['imgfile'] = argv[1]

    images = read_plainfile_tolist(ret['imgfile'])
    ret['imgid'] = images[0]
    if 'imgid' not in ret:
        print "ERROR: could not read imgid from input file"
        raise Exception("ERROR: could not read imgid from input file")

    if not is_image_analyzed(ret['imgid']):
        raise Exception("imageId ("+str(ret['imgid'])+") is not analyzed or analysis failed")

    ret['image_report'] = contexts['anchore_db'].load_image_report(ret['imgid'])

    ret['images'] = images

    ret['dirs'] = {}
    #ret['dirs']['datadir'] = argv[2]
    #ret['dirs']['imgdir'] = '/'.join([ret['dirs']['datadir'], ret['imgid'], 'image_output'])
    #ret['dirs']['analyzerdir'] = '/'.join([ret['dirs']['datadir'], ret['imgid'], 'analyzer_output'])
    #ret['dirs']['gatesdir'] = '/'.join([ret['dirs']['datadir'], ret['imgid'], 'gates_output'])

    ret['dirs']['outputdir'] = argv[3]

    try:
        ret['params'] = argv[4:]
    except:
        ret['params'] = list()

    for d in ret['dirs'].keys():
        thedir = ret['dirs'][d]
        if not os.path.exists(thedir):
            raise Exception(d + " directory '" + thedir + "' does not exist.")

    ret['meta'] = ret['image_report']['meta']
    ret['baseid'] = ret['image_report']['familytree'][0]

    ret['imgtags'] = ret['meta']['humanname']
    ret['output'] = '/'.join([ret['dirs']['outputdir'], ret['name']])
    ret['output_warns'] = '/'.join([ret['dirs']['outputdir'], ret['name']+".WARNS"])

    return (ret)

def get_docker_images(cli):
    ret = {}
    if not cli:
        return(ret)

    docker_images = cli.images(all=True)
    for i in docker_images:
        if 'Id' in i:
            Id = re.sub("sha256:", "", i['Id'])
            ret[Id] = i

    return(ret)

def anchore_common_context_setup(config):
    if 'docker_cli' not in contexts or not contexts['docker_cli']:

        dimages = {}
        try:
            contexts['docker_cli'] = docker.Client(base_url=config['docker_conn'], version='auto', timeout=int(config['docker_conn_timeout']))
            testconn = contexts['docker_cli'].version()
            dimages = get_docker_images(contexts['docker_cli']) 
        except Exception as err:
            contexts['docker_cli']=None

        contexts['docker_images'] = dimages

    if 'anchore_allimages' not in contexts or not contexts['anchore_allimages']:
        contexts['anchore_allimages'] = {}

    if 'anchore_db' not in contexts or not contexts['anchore_db']:
        contexts['anchore_db'] = anchore_image_db.load(driver=config['anchore_db_driver'], config=config)

    if 'anchore_auth' not in contexts or not contexts['anchore_auth']:
        aafile = os.path.join(config.config_dir, "anchore_auth.json")
        username = config.DEFAULT_ANON_ANCHORE_USERNAME
        password = config.DEFAULT_ANON_ANCHORE_PASSWORD
        if os.path.exists(aafile):
            try:
                with open(aafile, 'r') as FH:
                    aa = json.loads(FH.read())
                    username = aa['username']
                    password = aa['password']
            except:
                pass
                
        contexts['anchore_auth'] = anchore_auth.anchore_auth_init(username, password, aafile, config['anchore_client_url'], config['anchore_token_url'], config['anchore_auth_conn_timeout'], config['anchore_auth_max_retries'])

    if 'anchore_config' not in contexts or not contexts['anchore_config']:
        contexts['anchore_config'] = config

    return(True)

def load_analyzer_config(anchore_conf_dir):
    anchore_analyzer_config = {}
    csum = None

    anchore_analyzer_configfile = '/'.join([anchore_conf_dir, 'analyzer_config.yaml'])
    if os.path.exists(anchore_analyzer_configfile):
        try:
            with open(anchore_analyzer_configfile, 'r') as FH:
                adata = FH.read()
                csum = hashlib.md5(adata).hexdigest()
                anchore_analyzer_config = yaml.safe_load(adata)
        except Exception as err:
            raise err

    return(anchore_analyzer_config, csum)

# anchoreDB pass through functions

def is_image_analyzed(imageId):
    return(contexts['anchore_db'].is_image_analyzed(imageId))

def del_files_cache(imageId, namespace=None):
    return(contexts['anchore_db'].del_files_cache(imageId, namespace=None))

def load_files_tarfile(imageId, namespace):
    return(contexts['anchore_db'].load_files_tarfile(imageId, namespace))

def load_files_metadata(imageId, namespace):
    return(contexts['anchore_db'].load_files_metadata(imageId, namespace))

def load_files_namespaces(imageId):
    return(contexts['anchore_db'].load_files_namespaces(imageId))

def save_files(imageId, namespace, rootfsdir, files):
    return(contexts['anchore_db'].save_files(imageId, namespace, rootfsdir, files))

def save_gate_output(imageId, gate_name, data):
    return(contexts['anchore_db'].save_gate_output(imageId, gate_name, data))

def save_gate_help_output(gate_help):
    return(contexts['anchore_db'].save_gate_help_output(gate_help))

def save_analysis_output(imageId, module_name, module_value, data, module_type=None):
    return(contexts['anchore_db'].save_analysis_output(imageId, module_name, module_value, data, module_type=module_type))


def load_analysis_output(imageId, module_name, module_value):
    ret = {}
    ret = contexts['anchore_db'].load_analysis_output(imageId, module_name, module_value, module_type='user')
    if ret: return(ret)
    ret = contexts['anchore_db'].load_analysis_output(imageId, module_name, module_value, module_type='extra')
    if ret: return(ret)
    ret = contexts['anchore_db'].load_analysis_output(imageId, module_name, module_value)
    if ret: return(ret)

    return(ret)

def load_gate_output(imageId, gate_name):
    return(contexts['anchore_db'].load_gate_report(imageId, gate_name))


def load_image_report(imageId):
    return(contexts['anchore_db'].load_image_report(imageId))

def load_analysis_report(imageId):
    return(contexts['anchore_db'].load_analysis_report(imageId))

def load_gates_report(imageId):
    return(contexts['anchore_db'].load_gates_report(imageId))

def load_gates_eval_report(imageId):
    return(contexts['anchore_db'].load_gates_eval_report(imageId))


def list_analysis_outputs(imageId):
    return(contexts['anchore_db'].list_analysis_outputs(imageId))

def load_analyzer_manifest(imageId):
    return(contexts['anchore_db'].load_analyzer_manifest(imageId))


def load_image(imageId):
    return(contexts['anchore_db'].load_image(imageId))

def load_all_images():
    return(contexts['anchore_db'].load_all_images())


def is_image_analyzed(imageId):
    return(contexts['anchore_db'].is_image_analyzed(imageId))

def is_image_present(imageId, imagelist=None):
    return(contexts['anchore_db'].is_image_present(imageId, imagelist))


def get_image_list():
    return(contexts['anchore_db'].get_image_list())

def delete_image(imageId):
    return(contexts['anchore_db'].delete_image(imageId))


def is_intermediate_image(imageId, image_report=None):
    if not image_report:
        image_report = load_image_report(imageId)
    
    if image_report:
        try:
            if str(image_report['meta']['usertype']) in ['user', 'base', 'anchorebase', 'oldanchorebase']:
                return(False)
        except:
            pass

        try:
            if image_report['anchore_all_tags'] or image_report['anchore_current_tags']:
                return(False)
        except:
            pass

        try:
            if image_report['docker_data']['RepoTags']:
                return(False)
        except:
            pass
    else:
        raise Exception("could not load input image from anchoreDB: " + str(imageId))

    return(True)

def make_anchoretmpdir(tmproot):
    tmpdir = '/'.join([tmproot, str(random.randint(0, 9999999)) + ".anchoretmp"])
    try:
        os.makedirs(tmpdir)
        return(tmpdir)
    except:
        return(False)

def discover_gates():
    config = contexts['anchore_config']
    ret = {}

    gatesdir = '/'.join([config["scripts_dir"], "gates"])
    outputdir = make_anchoretmpdir(config['tmpdir'])

    path_overrides = ['/'.join([config['user_scripts_dir'], 'gates'])]
    if config['extra_scripts_dir']:
        path_overrides = path_overrides + ['/'.join([config['extra_scripts_dir'], 'gates'])]

    try:
        results = scripting.ScriptSetExecutor(path=gatesdir, path_overrides=path_overrides).execute(capture_output=True, fail_fast=True, cmdline=' '.join([outputdir, 'anchore_get_help']))
    except Exception as err:
        pass

    # walk through outputdir looking for dropped help output
    allhelp = {}
    for d in os.listdir(outputdir):
        gate_name = None
        match = re.match("(.*)\.help", d)
        if match:
            gate_name = match.group(1)
        if gate_name:
            helpfile = os.path.join(outputdir, d)
            with open(helpfile, 'r') as FH:
                helpdata = json.loads(FH.read())
            allhelp[gate_name] = helpdata

    shutil.rmtree(outputdir)

    save_gate_help_output(allhelp)

    return(allhelp)

def discover_from_info(dockerfile_contents):
    fromline = fromid = None
    fromline = re.match(".*FROM\s+(\S+).*", dockerfile_contents).group(1)
    if fromline:
        fromline = fromline.lower()
        if re.match("scratch", fromline) or re.match(".*<unknown>.*", fromline):
            fromid = fromline
        else:
            try:
                fromid = discover_imageId(fromline)
            except:
                fromid = None
    return(fromline, fromid)

def get_imageIds_named(name):
    ret = list()
    for result in contexts['anchore_db'].load_all_images_iter():
        imageId = result[0]
        image = result[1]
        if name == imageId:
            ret.append(imageId)
        elif re.match("^"+name, imageId):
            ret.append(imageId)
        elif name in image['anchore_all_tags'] + image['anchore_current_tags'] or name+":latest" in image['anchore_all_tags'] + image['anchore_current_tags']:
            ret.append(imageId)

    return(ret)

def discover_imageIds(namelist):
    ret = list()
    
    for name in namelist:
        result = discover_imageId(name)
        ret.append(result)

    return(ret)

def discover_imageId(name):

    ret = None

    # method -
    # 1) check if 'name' is in docker images list (key == imageId)
    # 2) check if 'name' or 'name:latest' is in docker images list repo/tags
    # 3) check anchoreDB
    # 4) check docker_inspect

    imageId = None
    try:
        _logger.debug("looking for name ("+name+") in docker_images")

        name_variants = []
        name_variants.append(name)

#        cleanname = name
#        for pre in ["", "sha256:", "docker.io/"]:
#            cleanname = re.sub("^"+pre, "", cleanname)
#            for post in ["", ":latest"]:
#                cleanname = re.sub(post+"$", "", cleanname)

#        for pre in ["", "sha256:", "docker.io/"]:
#            for post in ["", ":latest"]:
#                newname = pre+cleanname+post
#                if newname not in name_variants:
#                    name_variants.append(newname)

        try:
            docker_images = contexts['docker_images']
        except:
            docker_images = {}

        if name in docker_images.keys():
            imageId = name

        if not imageId:
            _logger.debug("looking for alternative names ("+name+") in docker_images")
            iname = re.sub("sha256:", "", name)
            for dimageId in docker_images.keys():
                i = docker_images[dimageId]
                if iname == i['Id'] or iname == re.sub("sha256:", "", i['Id']):
                    imageId = re.sub("sha256:", "", i['Id'])
                    break
                elif 'RepoTags' in i and i['RepoTags']:
                    for r in i['RepoTags']:
                        if name == r or name+":latest" == r:
                            imageId = re.sub("sha256:", "", i['Id'])
                            break
                        elif "docker.io/"+name == r or "docker.io/"+name+":latest" == r:
                            imageId = re.sub("sha256:", "", i['Id'])
                            break

        if not imageId:
            _logger.debug("looking for name ("+name+") in anchoreDB")
            for iname in name_variants:
                if contexts['anchore_db'].is_image_present(iname):
                    imageId = name
                    break

        if not imageId:
            _logger.debug("trying to load name ("+name+") from anchoreDB")
            for iname in name_variants:
                aimage = contexts['anchore_db'].load_image(iname)
                if aimage:
                    imageId = name
                    break

        if not imageId:
            _logger.debug("searching for name ("+name+") in anchoreDB")
            for iname in name_variants:
                ilist = get_imageIds_named(iname)
                if len(ilist) == 1:
                    imageId = ilist[0]
                    #aimage = contexts['anchore_db'].load_image(imageId)
                elif len(ilist) > 1:
                    raise ValueError("Input image name '"+str(iname)+"' is ambiguous in anchore:\n\tmatching imageIds: " + str(ilist))

        if not imageId:
            _logger.debug("trying docker.inspect_image on name ("+name+")")
            docker_cli = contexts['docker_cli']
            if docker_cli:
                try:
                    docker_data = docker_cli.inspect_image(name)
                    imageId = re.sub("sha256:", "", docker_data['Id'])
                except Exception as err:
                    pass
                    
    except ValueError as err:
        raise err

    except Exception as err:
        raise err

    if not imageId:
        raise ValueError("Input image name '"+str(name)+"' not found in local dockerhost or anchore DB.")

    return(imageId)

def get_images_from_kubectl():
    images = {}
    try:
        cmd = "kubectl get --all-namespaces --output json pods".split()
        jsonbuf = subprocess.check_output(cmd)
        kubedata = json.loads(jsonbuf)
        if 'items' in kubedata:
            for item in kubedata['items']:
                if 'status' in item:
                    if 'containerStatuses' in item['status']:
                        imagename = imageId = None
                        for cs in item['status']['containerStatuses']:
                            if 'image' in cs:
                                imagename = cs['image']
                            if 'imageID' in cs:
                                imageId = re.sub("^docker://sha256:", "", cs['imageID'])
                            
                            if imagename and imageId:
                                images[imageId] = imagename
    except Exception as err:
        raise err
    return(images)

def print_result(config, result, outputmode=None):
    if not result:
        return (False)

    if not outputmode:
        if config.cliargs['json']:
            outputmode = 'json'
        elif config.cliargs['plain']:
            outputmode = 'plaintext'
        elif config.cliargs['html']:
            outputmode = 'table'
            tablemode = 'html'
        else:
            outputmode = 'table'
            tablemode = 'stdout'

    if outputmode == 'table' and tablemode == 'stdout':
        try:
            width = int(subprocess.check_output(['stty', 'size'], stderr=open(os.devnull, 'w')).split()[1]) - 10
        except:
            width = 70
    else:
        width = 9999999
    
    if outputmode == 'json':
        print json.dumps(result)
    else:
        output = list()
        if len(result.keys()) > 0:
            sortby = False

            # this is awkward - need better way to differentiate header from results
            for i in result.keys():
                json_dict = result[i]
                sortby = False

                header = json_dict['result']['header']
                if outputmode == 'table':
                    header = [ re.sub("_", " ", x.encode('utf8')) for x in header ]
                    t = PrettyTable(header)
                    t.align = 'l'

                for h in header:
                    if re.match(r"^\*.*", h):
                        sortby = h

                break

            emptyresult = False
            for i in result.keys():
                json_dict = result[i]

                for orow in json_dict['result']['rows']:
                    if outputmode == 'table':
                        row = [ fill(x, max(12, width / (len(orow))) ).encode('utf8') for x in orow ]
                        t.add_row(row)
                    elif outputmode == 'plaintext':
                        row = [ re.sub("\s", ",", x.encode('utf8')) for x in orow ]
                        output.append(row)
                    elif outputmode == 'raw':
                        output.append(orow)

#            if outputmode == 'table' and tablemode == 'html':
#                print "<HTML><BODY>"

            if outputmode == 'table':
                if sortby:
                    if tablemode == 'stdout':
                        print t.get_string(sortby=sortby, reversesort=True)
                    elif tablemode == 'html':
                        print t.get_html_string(sortby=sortby, reversesort=True).encode('utf8')
                else:
                    if tablemode == 'stdout':
                        print t
                    elif tablemode == 'html':
                        print t.get_html_string().encode('utf8')
                print ""
            elif outputmode == 'plaintext':
                print ' '.join(header)
                print ""
                for r in output:
                    print ' '.join(r)
            elif outputmode == 'raw':
                for i in range(len(output)):
                    row = output[i]
                    for j in range(len(row)):
                        print "--- " + header[j] + " ---"
                        print output[i][j]
                        print 

        for k in result.keys():
            if 'warns' in result[k]:
                if outputmode == 'table':
                    t = PrettyTable(['WarningOutput'])
                    t.align = 'l'
                    for warn in result[k]['warns']:
                        t.add_row([warn])
                    
                    if tablemode == 'stdout':
                        print t
                    elif tablemode == 'html':
                        print "<BR></BR>"
                        print t.get_html_string()
                if outputmode == 'plaintext':
                    print "\nWarning Output\n"
                    for warn in result[k]['warns']:
                        print str(warn)
                if outputmode == 'raw':
                    pass

#        if outputmode == 'table' and tablemode == 'html':
#            print "</BODY></HTML>"
    return (True)

def apkg_parse_apkdb(apkdb):
    if not os.path.exists(apkdb):
        raise ValueError("cannot locate APK installed DB '"+str(apkdb)+"'")
        
    apkgs = {}                
    apkg = {
        'version':"N/A",
        'sourcepkg':"N/A",
        'release':"N/A",
        'origin':"N/A",
        'arch':"N/A",
        'license':"N/A",
        'size':"N/A"
    }
    thename = ""
    thepath = ""
    thefiles = list()
    allfiles = list()

    FH=open(apkdb, 'r')
    for l in FH.readlines():
        l = l.strip().decode('utf8')

        if not l:
            apkgs[thename] = apkg
            if thepath:
                flist = list()
                for x in thefiles:
                    flist.append(os.path.join(thepath, x))
                flist.append(os.path.join(thepath))
                allfiles = allfiles + flist
            apkgs[thename]['files'] = allfiles
            apkg = {
                'version':"N/A",
                'sourcepkg':"N/A",
                'release':"N/A",
                'origin':"N/A",
                'arch':"N/A",
                'license':"N/A",
                'size':"N/A",
                'type':"APKG"
            }
            allfiles = list()
            thefiles = list()
            thepath = ""

        patt = re.match("(\S):(.*)", l)
        if patt:
            (k, v) = patt.group(1,2)
            apkg['type'] = "APKG"
            if k == 'P':
                thename = v
                apkg['name'] = v
            elif k == 'V':
                vpatt = re.match("(\S*)-(\S*)", v)
                if vpatt:
                    (vers, rel) = vpatt.group(1, 2)
                else:
                    vers = v
                    rel = "N/A"                    
                apkg['version'] = vers
                apkg['release'] = rel
            elif k == 'm':
                apkg['origin'] = v
            elif k == 'I':
                apkg['size'] = v
            elif k == 'L' and v:
                apkg['license'] = v
            elif k == 'o':
                apkg['sourcepkg'] = v
            elif k == 'A':
                apkg['arch'] = v
            elif k == 'F':
                if thepath:
                    flist = list()
                    for x in thefiles:
                        flist.append(os.path.join(thepath, x))
                    flist.append(os.path.join(thepath))
                    allfiles = allfiles + flist

                thepath = "/" + v
                thefiles = list()
            elif k == 'R':
                thefiles.append(v)

    FH.close()
    return(apkgs)

def apkg_get_all_pkgfiles(unpackdir):
    apkdb = '/'.join([unpackdir, 'rootfs/lib/apk/db/installed'])
    return(apkg_parse_apkdb(apkdb))

def dpkg_compare_versions(v1, op, v2):
    cmd = ['dpkg', '--compare-versions', v1, op, v2]
    return(subprocess.call(cmd))

def apkg_compare_versions(v1, op, v2):

    if op == 'eq':
        if v1 == v2:
            return(0);
        else:
            return(1);

    elif op == 'lt':
        if v1 < v2:
            return(0)
        else:
            return(1)
    elif op == 'gt':
        if v1 > v2:
            return(0)
        else:
            return(1)

    _logger.error("invalid op specified in compare: " + str(op))
    return(1)

def dpkg_get_all_packages(unpackdir):
    actual_packages = {}
    all_packages = {}
    other_packages = {}
    cmd = ["dpkg-query", "--admindir="+unpackdir+"/rootfs/var/lib/dpkg", "-W", "-f="+"${Package} ${Version} ${source:Package} ${source:Version} ${Architecture}\\n"]
    try:
        sout = subprocess.check_output(cmd)
        for l in sout.splitlines(True):
            l = l.strip()
            l = l.decode('utf8')
            (p, v, sp, sv, arch) = re.match('(\S*)\s*(\S*)\s*(\S*)\s*(\S*)\s*(.*)', l).group(1, 2, 3, 4, 5)
            if p and v:
                if p not in actual_packages:
                    actual_packages[p] = {'version':v, 'arch':arch}
                if p not in all_packages:
                    all_packages[p] = {'version':v, 'arch':arch}
            if sp and sv:
                if sp not in all_packages:
                    all_packages[sp] = {'version':sv, 'arch':arch}
            if p and v and sp and sv:
                if p == sp and v != sv:
                    other_packages[p] = [{'version':sv, 'arch':arch}]

    except Exception as err:
        print "Could not run command: " + str(cmd)
        print "Exception: " + str(err)
        print "Please ensure the command 'dpkg' is available and try again"
        raise err

    ret = (all_packages, actual_packages, other_packages)
    return(ret)

def dpkg_get_all_pkgfiles(unpackdir):
    allfiles = {}

    try:
        (allpkgs, actpkgs, othpkgs) = dpkg_get_all_packages(unpackdir)    
        cmd = ["dpkg-query", "--admindir="+unpackdir+"/rootfs/var/lib/dpkg", "-L"] + actpkgs.keys()
        sout = subprocess.check_output(cmd)
        for l in sout.splitlines():
            l = l.strip()
            l = l.decode('utf8')
            allfiles[l] = True
            
    except Exception as err:
        print "Could not run command: " + str(' '.join(cmd))
        print "Exception: " + str(err)
        print "Please ensure the command 'dpkg' is available and try again"
        raise err

    return(allfiles)

def rpm_prepdb(unpackdir):
    origrpmdir = os.path.join(unpackdir, 'rootfs', 'var', 'lib', 'rpm')
    ret = origrpmdir

    if os.path.exists(origrpmdir):
        newrpmdirbase = make_anchoretmpdir(unpackdir)
        newrpmdir = os.path.join(newrpmdirbase, 'var', 'lib', 'rpm')
        try:
            shutil.copytree(origrpmdir, newrpmdir)
            sout = subprocess.check_output(['rpmdb', '--root='+newrpmdirbase, '--dbpath=/var/lib/rpm', '--rebuilddb'])
            ret = newrpmdir
        except:
            pass

    return(ret)

def rpm_get_all_packages(unpackdir):
    rpms = {}
    rpmdbdir = rpm_prepdb(unpackdir)
    try:
        sout = subprocess.check_output(['rpm', '--dbpath='+rpmdbdir, '--queryformat', '%{NAME} %{VERSION} %{RELEASE} %{ARCH}\n', '-qa'], stderr=subprocess.STDOUT)
        for l in sout.splitlines():
            l = l.strip()
            l = l.decode('utf8')
            (name, vers, rel, arch) = re.match('(\S*)\s*(\S*)\s*(\S*)\s*(.*)', l).group(1, 2, 3, 4)
            rpms[name] = {'version':vers, 'release':rel, 'arch':arch}
    except Exception as err:
        print err.output
        raise ValueError("could not get package list from RPM database: " + str(err))

    return(rpms)

def rpm_get_all_pkgfiles(unpackdir):
    rpmfiles = {}
    rpmdbdir = rpm_prepdb(unpackdir)
    try:
        sout = subprocess.check_output(['rpm', '--dbpath='+rpmdbdir, '-qal'])
        for l in sout.splitlines():
            l = l.strip()
            l = l.decode('utf8')
            rpmfiles[l] = True
    except Exception as err:
        raise ValueError("could not get file list from RPM database: " + str(err))

    return(rpmfiles)

def gem_parse_meta(gem):
    ret = {}

    name = None
    versions = []
    lics = []
    latest = None
    origins = []
    sourcepkg = None
    rfiles = []

    try:
        for line in gem.splitlines():
            line = line.strip()
            line = re.sub("\.freeze", "", line)

            # look for the unicode \u{} format and try to convert to something python can use
            try:
                replline = line
                mat = "\\\u{.*?}"
                patt = re.match(r".*("+mat+").*", replline)
                while(patt):
                    replstr = ""
                    subpatt = re.match("\\\u{(.*)}", patt.group(1))
                    if subpatt:
                        chars = subpatt.group(1).split()
                        for char in chars:
                            replstr += unichr(int(char, 16))

                    if replstr:
                        replline = re.sub(re.escape(patt.group(1)), replstr, replline, 1)

                    patt = re.match(r".*("+mat+").*", replline)
                    line = replline
            except Exception as err:
                pass

            patt = re.match(".*\.name *= *(.*) *", line)
            if patt:
                name = json.loads(patt.group(1))

            patt = re.match(".*\.homepage *= *(.*) *", line)
            if patt:
                sourcepkg = json.loads(patt.group(1))

            patt = re.match(".*\.version *= *(.*) *", line)
            if patt:
                v = json.loads(patt.group(1))
                latest = v
                versions.append(latest)

            patt = re.match(".*\.licenses *= *(.*) *", line)
            if patt:
                lstr = re.sub("^\[|\]$", "", patt.group(1)).split(',')
                for thestr in lstr:
                    thestr = re.sub(' *" *', "", thestr)
                    lics.append(thestr)

            patt = re.match(".*\.authors *= *(.*) *", line)
            if patt:
                lstr = re.sub("^\[|\]$", "", patt.group(1)).split(',')
                for thestr in lstr:
                    thestr = re.sub(' *" *', "", thestr)
                    origins.append(thestr)

            patt = re.match(".*\.files *= *(.*) *", line)
            if patt:
                lstr = re.sub("^\[|\]$", "", patt.group(1)).split(',')
                for thestr in lstr:
                    thestr = re.sub(' *" *', "", thestr)
                    rfiles.append(thestr)

    except Exception as err:
        print "WARN could not fully parse gemspec file: " + str(name) + ": exception: " + str(err)
        return({})

    if name:
        ret[name] = {'name':name, 'lics':lics, 'versions':versions, 'latest':latest, 'origins':origins, 'sourcepkg':sourcepkg, 'files':rfiles}

    return(ret)

def npm_parse_meta(npm):

    record = {}

    name = npm.pop('name', None)
    if not name:
        return(record)

    lics = list()
    versions = list()
    latest = None
    origins = list()
    sourcepkg = None

    npmtime = npm.pop('time', None)
    npmdesc = npm.pop('description', None)
    npmdisttags = npm.pop('dist-tags', None)
    npmkeywords = npm.pop('keywords', None)


    npmlicense = npm.pop('license', None)
    npmversions = npm.pop('versions', None)
    npmversion = npm.pop('version', None)
    npmauthor = npm.pop('author', None)
    npmmaintainers = npm.pop('maintainers', None)
    npmrepository = npm.pop('repository', None)
    npmhomepage= npm.pop('homepage', None)

    if npmlicense:
        if isinstance(npmlicense, basestring):
            lics.append(npmlicense)
        elif isinstance(npmlicense, dict):
            for ktype in ['type', 'name', 'license', 'sourceType']:
                lic = npmlicense.pop(ktype, None)
                if lic:
                    lics.append(lic)
        elif isinstance(npmlicense, list):
            for lentry in npmlicense:
                if isinstance(lentry, basestring):
                    lics.append(lentry)
                elif isinstance(lentry, dict):
                    for ktype in ['type', 'name', 'license', 'sourceType']:
                        lic = lentry.pop(ktype, None)
                        if lic:
                            lics.append(lic)
        else:
            print "unknown type (" + str(name) + "): " + str(type(npmlicense))


    if npmversions:
        if isinstance(npmversions, dict):
            versions = npmversions.keys()
            for v in npmversions:
                if npmversions[v] == 'latest':
                    latest = v
        elif isinstance(npmversions, list):
            versions = npmversions
    elif npmversion:
        versions.append(npmversion)

    astring = None
    if npmauthor:
        if isinstance(npmauthor, basestring):
            astring = npmauthor
        elif isinstance(npmauthor, dict):
            aname = npmauthor.pop('name', None)
            aurl = npmauthor.pop('url', None)
            if aname:
                astring = aname
                if aurl:
                    astring += " ("+aurl+")"
        else:
            print "unknown type (" + str(name) + "): "+ str(type(npmauthor))

    elif npmmaintainers:
        for m in npmmaintainers:
            aname = m.pop('name', None)
            aemail = m.pop('email', None)
            if aname:
                astring = aname
                if aemail:
                    astring += " ("+aemail+")"

    if astring:
        origins.append(astring)

    if npmrepository:
        if isinstance(npmrepository, dict):
            sourcepkg = npmrepository.pop('url', None)
        elif isinstance(npmrepository, basestring):
            sourcepkg = npmrepository
        else:
            print "unknown type (" + str(name) + "): " + str(type(npmrepository))

    elif npmhomepage:
        if isinstance(npmhomepage, basestring):
            sourcepkg = npmhomepage

    if not lics:
        print "WARN: ("+name+") no lics: " + str(npm)
    if not versions:
        print "WARN: ("+name+") no versions: " + str(npm)
    if not origins:
        print "WARN: ("+name+") no origins: " + str(npm)
    if not sourcepkg:
        print "WARN: ("+name+") no sourcepkg: " + str(npm)

    if name:
        record[name] = {'name':name, 'lics':lics, 'versions':versions, 'latest':latest, 'origins':origins, 'sourcepkg':sourcepkg}

    return(record)

def get_distro_from_imageId(imageId):
    meta = {
        'DISTRO':None,
        'DISTROVERS':None,
        'LIKEDISTRO':None
    }

    anchore_analyzer_meta = load_analysis_output(imageId, 'analyzer_meta', 'analyzer_meta')
    meta['DISTRO'] = anchore_analyzer_meta.pop('DISTRO', 'UNKNOWN')
    meta['DISTROVERS'] = anchore_analyzer_meta.pop('DISTROVERS', 'UNKNOWN')
    meta['LIKEDISTRO'] = anchore_analyzer_meta.pop('LIKEDISTRO', 'UNKNOWN')
    return (meta)

def get_distro_from_path(inpath):

    meta = {
        'DISTRO':None,
        'DISTROVERS':None,
        'LIKEDISTRO':None
    }

    if os.path.exists('/'.join([inpath,"/etc/os-release"])):
        FH=open('/'.join([inpath,"/etc/os-release"]), 'r')
        for l in FH.readlines():
            l = l.strip()
            l = l.decode('utf8')
            try:
                (key, val) = l.split("=")
                val = re.sub(r'"', '', val)
                if key == "ID":
                    meta['DISTRO'] = val
                elif key == "VERSION_ID":
                    meta['DISTROVERS'] = val
                elif key == "ID_LIKE":
                    meta['LIKEDISTRO'] = ','.join(val.split())
            except:
                a=1
        FH.close()
    elif os.path.exists('/'.join([inpath, "/etc/system-release-cpe"])):
        FH=open('/'.join([inpath, "/etc/system-release-cpe"]), 'r')
        for l in FH.readlines():
            l = l.strip()
            l = l.decode('utf8')
            try:
                distro = l.split(':')[2]
                vers = l.split(':')[4]
                meta['DISTRO'] = distro
                meta['DISTROVERS'] = vers
            except:
                pass
        FH.close()
    elif os.path.exists('/'.join([inpath, "/etc/redhat-release"])):
        FH=open('/'.join([inpath, "/etc/redhat-release"]), 'r')
        for l in FH.readlines():
            l = l.strip()
            l = l.decode('utf8')
            try:
                distro = vers = None
                patt = re.match(".*CentOS.*", l)
                if patt:
                    distro = 'centos'

                patt = re.match(".*(\d+\.\d+).*", l)
                if patt:
                    vers = patt.group(1)

                if distro:
                    meta['DISTRO'] = distro
                if vers:
                    meta['DISTROVERS'] = vers
            except:
                pass
        FH.close()
    elif os.path.exists('/'.join([inpath, "/bin/busybox"])):
        meta['DISTRO'] = "busybox"
        try:
            sout = subprocess.check_output(['/'.join([inpath, "/bin/busybox"])])
            fline = sout.splitlines(True)[0]
            slist = fline.split()
            meta['DISTROVERS'] = slist[1]
        except:
            meta['DISTROVERS'] = "0"
    elif os.path.exists('/'.join([inpath, "/etc/debian_version"])):
        with open('/'.join([inpath, "/etc/debian_version"]), 'r') as FH:
            meta['DISTRO'] = 'debian'
            for line in FH.readlines():
                line = line.strip()
                patt = re.match("(\d+)\..*", line)
                if patt:
                    meta['DISTROVERS'] = patt.group(1)

    if not meta['DISTRO']:
        meta['DISTRO'] = "Unknown"
    if not meta['DISTROVERS']:
        meta['DISTROVERS'] = "0"
    if not meta['LIKEDISTRO']:
        meta['LIKEDISTRO'] = meta['DISTRO']

    return(meta)

def get_distro_flavor(distro, version, likedistro=None):
    ret = {
        'flavor':'Unknown',
        'version':'0',
        'fullversion':version,
        'distro':distro,
        'likedistro':distro,
        'likeversion':version
    }

    if distro in ['centos', 'rhel', 'fedora']:
        ret['flavor'] = "RHEL"
        ret['likedistro'] = 'centos'
    elif distro in ['debian', 'ubuntu']:
        ret['flavor'] = "DEB"
    elif distro in ['busybox']:
        ret['flavor'] = "BUSYB"
    elif distro in ['alpine']:
        ret['flavor'] = "ALPINE"
    elif distro in ['ol']:
        ret['flavor'] = "RHEL"
        ret['likedistro'] = 'centos'

    if ret['flavor'] == 'Unknown' and likedistro:
        likedistros = likedistro.split(',')
        for distro in likedistros:
            if distro in ['centos', 'rhel', 'fedora']:
                ret['flavor'] = "RHEL"
                ret['likedistro'] = 'centos'
            elif distro in ['debian', 'ubuntu']:
                ret['flavor'] = "DEB"
            elif distro in ['busybox']:
                ret['flavor'] = "BUSYB"
            elif distro in ['alpine']:
                ret['flavor'] = "ALPINE"
            elif distro in ['ol']:
                ret['flavor'] = "RHEL"
                ret['likedistro'] = 'centos'

            if ret['flavor'] != 'Unknown':
                break

    patt = re.match("(\d*)\.*(\d*)", version)
    if patt:
        (vmaj, vmin) = patt.group(1,2)
        if vmaj:
            ret['version'] = vmaj
            ret['likeversion'] = vmaj

    patt = re.match("(\d+)\.*(\d+)\.*(\d+)", version)
    if patt:
        (vmaj, vmin, submin) = patt.group(1,2,3)
        if vmaj and vmin:
            ret['version'] = vmaj + "." + vmin
            ret['likeversion'] = vmaj + "." + vmin

    return(ret)

def cve_load_data_imageId(imageId, cve_data_context=None):
    cve_data = None
    
    distrometa = get_distro_from_imageId(imageId)

    idistro = distrometa['DISTRO']
    idistrovers = distrometa['DISTROVERS']

    distrodict = get_distro_flavor(idistro, idistrovers)

    distro = distrodict['distro']
    distrovers = distrodict['version']
    likedistro = distrodict['likedistro']
    likeversion = distrodict['likeversion']
    fulldistro = distrodict['distro']
    fullversion = distrodict['fullversion']
    
    distrolist = [(distro,distrovers), (likedistro, likeversion), (fulldistro, fullversion)]
    for f in distrolist:
        dstr = ':'.join([f[0], f[1]])
        if cve_data_context and dstr in cve_data_context:
            cve_data = cve_data_context[dstr]
            break
        else:
            feeddata = anchore_feeds.load_anchore_feed('vulnerabilities', ':'.join([f[0], f[1]]))
            if feeddata['success']:
                cve_data = feeddata['data']
                if cve_data_context != None and dstr not in cve_data_context:
                    cve_data_context[dstr] = cve_data
                break

    if not cve_data:
        raise ValueError("cannot find CVE data associated with the input container distro: ("+str(distrolist)+")")

    return (dstr, cve_data)

def cve_load_data(image, cve_data_context=None):

    cve_data = None
    imageId = image.meta['imageId']

    (dst, ret) = cve_load_data_imageId(imageId, cve_data_context=cve_data_context)

    return(ret)

def cve_scanimages(images, pkgmap, flavor, cve_data):
    results = {}
    for v in cve_data:
        outel = {}
        vuln = v['Vulnerability']

        #print "cve-scan: VULN NAME CVE: " + vuln['Name']
        for vtag in ['FixedIn', 'VulnerableIn']:
            if vtag in vuln:
                for fixes in vuln[vtag]:
                    isvuln = False
                    vpkg = fixes['Name']
                    #print "cve-scan: Vulnerable Package: " + vpkg
                    if vpkg in pkgmap:
                        for ivers in pkgmap[vpkg]['versions'].keys():
                            vvers = re.sub(r'^[0-9]*:', '', fixes['Version'])
                            #print "cve-scan: " + vpkg + "\n\tfixed vulnerability package version: " + vvers + "\n\timage package version: " + ivers

                            iversonly = ivers
                            isvuln = is_pkg_vuln(vtag, vpkg, flavor, ivers, iversonly, vvers)

                            if isvuln:
                                #print "cve-scan: Found vulnerable package: " + vpkg
                                severity = url = description = 'Not Available'
                                if 'Severity' in vuln:
                                    severity = vuln['Severity']
                                if 'Link' in vuln:
                                    url = vuln['Link']
                                if 'Description' in vuln:
                                    description = vuln['Description']

                                outel = {'images':pkgmap[vpkg]['versions'][ivers], 'pkgName': vpkg, 'imageVers': ivers, 'fixVers': vvers, 'severity': severity, 'url': url, 'description': description}

            if outel:
                results[vuln['Name']] = outel

    return(results)

def normalize_packages(imageId):
    
    distrometa = get_distro_from_imageId(imageId)
    idistro = distrometa['DISTRO']
    idistrovers = distrometa['DISTROVERS']
    distrodict = get_distro_flavor(idistro, idistrovers)
    flavor = distrodict['flavor']

    ret = {
        'bin_packages':{},
        'bin_to_src':{},
        'src_to_bin':{}
    }

    try:
        all_packages_detail = load_analysis_output(imageId, 'package_list', 'pkgs.allinfo')
        if not all_packages_detail:
            raise Exception("no package detail")

        for pkg in all_packages_detail.keys():

            try:
                # copy package details
                data = json.loads(all_packages_detail[pkg])
                ret['bin_packages'][pkg] = {}
                ret['bin_packages'][pkg].update(data)
                if data['release'] and data['release'] != "N/A":
                    ret['bin_packages'][pkg]['fullvers'] = data['version']+"-"+data['release']
                else:
                    ret['bin_packages'][pkg]['fullvers'] = data['version']

                # map binary to source pkg
                if pkg not in ret['bin_to_src']:
                    ret['bin_to_src'][pkg] = []

                # need to clean out/remove the "+b[0-9]+" from DEBs
                if flavor == 'DEB':
                    cleanvers = re.sub(re.escape("+b")+"\d+.*", "", data['version'])
                    spkg = re.sub(re.escape("-"+cleanvers), "", data['sourcepkg'])
                else:
                    spkg = re.sub(re.escape("-"+data['version'])+".*", "", data['sourcepkg'])

                ret['bin_to_src'][pkg].append(spkg)

                # map source pkg to binary
                if spkg not in ret['src_to_bin']:
                    ret['src_to_bin'][spkg] = []
                ret['src_to_bin'][spkg].append(pkg)
            except Exception as err:
                errmsg = "failed to normalize package: " + str(pkg) + " - exception: " + str(err)
                _logger.error(errmsg)
            
        return(ret)
    except Exception as err:
        _logger.debug("no package detail analyzer output found, falling back to base package analyzer output")

    try:
        all_packages = load_analysis_output(imageId, 'package_list', 'pkgs.all')
        all_packages_plus_source = load_analysis_output(imageId, 'package_list', 'pkgs_plus_source.all')
        
        all_packages.update(all_packages_plus_source)
        if not all_packages:
            raise Exception("no package data")

        for pkg in all_packages.keys():

            if flavor == 'RHEL':
                fname = pkg + '-' + all_packages[pkg] + '.tmparch.rpm'
                (n,v,r,e,a) = splitFilename(fname)
                el = {'version':v, 'release':r, 'fullvers':all_packages[pkg], 'type':'RPM', 'origin':"N/A", 'sourcepkg':"N/A", 'license':"N/A", 'arch':"N/A", 'size':"N/A"}
                ret['bin_packages'][pkg] = el
            elif flavor == 'DEB':
                try:
                    (v, r) = all_packages[pkg].split("-")
                except:
                    v = all_packages[pkg]
                    r = None
                    
                if r:
                    el = {'version':v, 'release':r, 'fullvers':all_packages[pkg], 'type':'DPKG', 'origin':"N/A", 'sourcepkg':"N/A", 'license':"N/A", 'arch':"N/A", 'size':"N/A"}
                else:
                    el = {'version':v, 'release':"", 'fullvers':all_packages[pkg], 'type':'DPKG', 'origin':"N/A", 'sourcepkg':"N/A", 'license':"N/A", 'arch':"N/A", 'size':"N/A"}

                ret['bin_packages'][pkg] = el
            elif flavor == 'ALPINE':
                try:
                    (v, r) = all_packages[pkg].split("-")
                except:
                    v = all_packages[pkg]
                    r = None
                    
                if r:
                    el = {'version':v, 'release':r, 'fullvers':all_packages[pkg], 'type':'APKG', 'origin':"N/A", 'sourcepkg':"N/A", 'license':"N/A", 'arch':"N/A", 'size':"N/A"}
                else:
                    el = {'version':v, 'release':"", 'fullvers':all_packages[pkg], 'type':'APKG', 'origin':"N/A", 'sourcepkg':"N/A", 'license':"N/A", 'arch':"N/A", 'size':"N/A"}

                ret['bin_packages'][pkg] = el

            else:
                pass

    except:
        _logger.debug("no package data found, skipping")

    try:
        all_packages_plus_source = load_analysis_output(imageId, 'package_list', 'pkgs_plus_source.all')
        for pkg in all_packages_plus_source.keys():
            if pkg not in ret['bin_packages']:
                svers = all_packages_plus_source[pkg]

                for rpkg in ret['bin_packages'].keys():
                    check_full = rpkg+"-"+all_packages_plus_source[pkg]
                    if ret['bin_packages'][rpkg]['fullname'] == check_full:

                        if rpkg not in ret['bin_to_src']:
                            ret['bin_to_src'][rpkg] = []
                        ret['bin_to_src'][rpkg].append(pkg)
                        
                        if pkg not in ret['src_to_bin']:
                            ret['src_to_bin'][pkg] = []
                        ret['src_to_bin'][pkg].append(rpkg)

    except Exception as err:
        _logger.debug("no source package data found, skipping")

    return(ret)

def cve_scan_packages(cve_data, norm_packages, flavor):
    import time
    #start = time.time()
    results = {}
    for v in cve_data:
        vuln = v['Vulnerability']
        #print "cve-scan: CVE: " + vuln['Name']

        fixedIn = {}
        if 'FixedIn' in vuln:
            for fixes in vuln['FixedIn']:
                vpkgname = fixes['Name']
                fixVers = re.sub(r'^[0-9]*:', '', fixes['Version'])
                fixedIn[vpkgname] = fixVers

        if 'VulnerableIn' in vuln:
            for vulns in vuln['VulnerableIn']:
                vpkgname = vulns['Name']
                vulnVers = re.sub(r'^[0-9]*:', '', vulns['Version'])
                if vpkgname not in fixedIn:
                    fixedIn[vpkgname] = "None"

        for vtag in ['FixedIn', 'VulnerableIn']:
            if vtag in vuln:
                for fixes in vuln[vtag]:
                    vpkgname = fixes['Name']
                    vvers = re.sub(r'^[0-9]*:', '', fixes['Version'])

                    fixVers = fixedIn[vpkgname]

                    vpkgs = []
                    ivers = iversonly = irelonly = None

                    #print "cve-scan: Vulnerable Package: " + vpkgname

                    if vpkgname in norm_packages['bin_packages']:
                        ivers = norm_packages['bin_packages'][vpkgname]['fullvers']
                        iversonly = norm_packages['bin_packages'][vpkgname]['version']
                        irelonly = norm_packages['bin_packages'][vpkgname]['release']
                        vpkgs = [vpkgname]
                    elif vpkgname in norm_packages['src_to_bin']:
                        bpkg = norm_packages['src_to_bin'][vpkgname][0]
                        ivers = norm_packages['bin_packages'][bpkg]['fullvers']
                        iversonly = norm_packages['bin_packages'][bpkg]['version']
                        irelonly = norm_packages['bin_packages'][bpkg]['release']
                        vpkgs = norm_packages['src_to_bin'][vpkgname]
                    else:
                        # cve vuln package cannot be mapped to anything installed
                        pass


                    # go through all found packages that mapped to the CVE vul package to check versions for vulnerability
                    for vpkg in vpkgs:
                        isvuln = False

                        isvuln = is_pkg_vuln(vtag, vpkg, flavor, ivers, iversonly, vvers)

                        # finally - format and add result to return dict if vulnerability has been determined

                        if isvuln:
                            #print "cve-scan: Found vulnerable package: " + vpkg

                            severity = url = description = 'Not Available'
                            if 'Severity' in vuln:
                                severity = vuln['Severity']
                            if 'Link' in vuln:
                                url = vuln['Link']
                            if 'Description' in vuln:
                                description = vuln['Description']

                            outel = {'pkgName': vpkg, 'imageVers': ivers, 'fixVers': fixVers, 'severity': severity, 'url': url, 'description': description}

                            if vuln['Name'] not in results:
                                results[vuln['Name']] = []
                            if outel not in results[vuln['Name']]:
                                results[vuln['Name']].append(outel)

    return (results)

def cve_scanimage(cve_data, imageId):
    if not cve_data:
        return ({})

    try:
        distrometa = get_distro_from_imageId(imageId)
        idistro = distrometa['DISTRO']
        idistrovers = distrometa['DISTROVERS']
        distrodict = get_distro_flavor(idistro, idistrovers)
        flavor = distrodict['flavor']
    except Exception as err:
        print "cve-scan: could not determine image distro: returning empty value"
        return({})
    
    import time

    norm_packages = normalize_packages(imageId)

    if 'bin_packages' not in norm_packages or not norm_packages['bin_packages']:
        return({})

    results = cve_scan_packages(cve_data, norm_packages, flavor)

    return (results)

def is_pkg_vuln(vtag, vpkg, flavor, ivers, iversonly, vvers):
    isvuln = False
    #print "cve-scan: " + vpkg + "\n\tvulnerability package version: " + vvers + "\n\timage package version: " + ivers

    if vtag == 'VulnerableIn' and vvers == 'all':
        isvuln = True
    elif vvers != 'None':
        if flavor == 'RHEL':
            fixfile = vpkg + "-" + vvers + ".arch.rpm"
            imagefile = vpkg + "-" + ivers + ".arch.rpm"
            (n1, v1, r1, e1, a1) = splitFilename(imagefile)
            (n2, v2, r2, e2, a2) = splitFilename(fixfile)
            if vtag == 'FixedIn':
                if rpm.labelCompare(('1', v1, r1), ('1', v2, r2)) < 0:
                    isvuln = True
            elif vtag == 'VulnerableIn':
                if ivers == vvers or iversonly == vvers:
                    isvuln = True

        elif flavor == 'DEB':
            if vtag == 'FixedIn':
                if ivers != vvers:
                    comp_rc = dpkg_compare_versions(ivers, 'lt', vvers)
                    if comp_rc == 0:
                        isvuln = True
            elif vtag == 'VulnerableIn':
                if ivers == vvers or iversonly == vvers:
                    isvuln = True

        elif flavor == "ALPINE":
            if vtag == 'FixedIn':
                comp_rc = apkg_compare_versions(ivers, 'lt', vvers)
                if comp_rc == 0:
                    isvuln = True
            elif vtag == 'VulnerableIn':
                if ivers == vvers or iversonly == vvers:
                    isvuln = True
    else:
        isvuln = True

    return(isvuln)

def image_context_add(imagelist, allimages, docker_cli=None, dockerfile=None, anchore_datadir=None, tmproot='/tmp', anchore_db=None, docker_images=None, must_be_analyzed=False, usertype=None, must_load_all=False):
    retlist = list()
    for i in imagelist:
        if i in allimages:
            retlist.append(i)
        elif must_be_analyzed and not anchore_db.is_image_analyzed(i):
            errorstr = "Image(s) must be analyzed before operation can be performed.\n\tImage: " + str(i)
            raise Exception(errorstr)
        else:
            try:
                newimage = anchore_image.AnchoreImage(i, anchore_datadir, docker_cli=docker_cli, allimages=allimages, dockerfile=dockerfile, tmpdirroot=tmproot, usertype=usertype, anchore_db=anchore_db, docker_images=docker_images)
            except Exception as err:
                if must_load_all:
                    traceback.print_exc()
                    errorstr = "Could not load/initialize all input images.\n" + "\tImage: " + str(i) + "\n\tInfo: " + str(err.message)
                    raise Exception(errorstr)

            if not must_be_analyzed or newimage.is_analyzed():
                allimages[newimage.meta['imageId']] = newimage
                retlist.append(newimage.meta['imageId'])

            if must_be_analyzed and not newimage.is_analyzed():
                errorstr = "Image(s) must be analyzed before operation can be performed.\n\tImage: " + str(i)
                raise Exception(errorstr)

    return (retlist)


def diff_images(imageId, baseimageId):
    ret = {}

    areport = contexts['anchore_db'].load_analysis_report(imageId)
    breport = contexts['anchore_db'].load_analysis_report(baseimageId)
    
    for module_name in areport.keys():
        if module_name in breport:
            for module_value in areport[module_name].keys():
                if module_value in breport[module_name]:
                    for module_type in areport[module_name][module_value].keys():
                        output = {}

                        adata = areport[module_name][module_value][module_type]
                        bdata = breport[module_name][module_value][module_type]

                        for akey in adata.keys():
                            if akey not in bdata:
                                output[akey] = "INIMG_NOTINBASE"
                            elif adata[akey] != bdata[akey]:
                                output[akey] = "VERSION_DIFF"
                        for bkey in bdata.keys():
                            if bkey not in adata:
                                output[bkey] = "INBASE_NOTINIMG"
                        if module_name not in ret:
                            ret[module_name] = {}
                        if module_value not in ret[module_name]:
                            ret[module_name][module_value] = {}

                        ret[module_name][module_value][module_type] = output

    return(ret)

def update_file_list(listbuf, outfile, backup=False):
    src = listbuf
    if not os.path.exists(outfile):
        write_plainfile_fromlist(outfile, src)
    else:
        dst = read_plainfile_tolist(outfile)
        if src != dst:
            if backup:
                hfile = outfile + "." + str(int(time.time()))
                os.rename(outfile, hfile)

            write_plainfile_fromlist(outfile, src)

    return (True)


def update_file_jsonstr(jsonbuf, outfile, backup=False):
    src = json.loads(jsonbuf)
    if not os.path.exists(outfile):
        FH = open(outfile, 'w')
        FH.write(json.dumps(src))
        FH.close()
    else:
        FH = open(outfile, 'r')
        dst = json.loads(FH.read())
        FH.close()
        if src != dst:
            if backup:
                hfile = outfile + "." + str(int(time.time()))
                os.rename(outfile, hfile)
            FH = open(outfile, 'w')
            FH.write(json.dumps(src))
            FH.close()

    return (True)


def update_file_str(buf, outfile, backup=False):
    src = buf
    if not os.path.exists(outfile):
        write_plainfile_fromstr(outfile, src)
    else:
        dst = read_plainfile_tostr(outfile)
        if src != dst:
            if backup:
                hfile = outfile + "." + str(int(time.time()))
                os.rename(outfile, hfile)
            write_plainfile_fromstr(outfile, src)
    return (True)


def write_plainfile_fromstr(file, instr):
    FH=open(file, 'w')
    thestr = instr.encode('utf8')
    FH.write(thestr)
    FH.close()

def read_plainfile_tostr(file):
    if not os.path.isfile(file):
        return ("")
    FH = open(file, 'r')
    ret = FH.read().decode('utf8')
    FH.close()
    return (ret)


def read_kvfile_tolist(file):
    if not os.path.isfile(file):
        return([])

    ret = list()
    FH=open(file, 'r')
    for l in FH.readlines():
        l = l.strip().decode('utf8')
        if l:
            row = l.split()
            for i in range(0, len(row)):
                row[i] = re.sub("____", " ", row[i])
            ret.append(row)
    FH.close()

    return (ret)

def read_plainfile_tolist(file):
    if not os.path.isfile(file):
        return([])

    ret = list()
    FH=open(file, 'r')
    for l in FH.readlines():
        l = l.strip().decode('utf8')
        if l:
            ret.append(l)
    FH.close()

    return (ret)

def read_kvfile_todict(file):
    if not os.path.isfile(file):
        return ({})

    ret = {}
    FH = open(file, 'r')
    for l in FH.readlines():
        l = l.strip().decode('utf8')
        if l:
            (k, v) = re.match('(\S*)\s*(.*)', l).group(1, 2)
            k = re.sub("____", " ", k)
            ret[k] = v
    FH.close()

    return (ret)

def write_plainfile_fromlist(file, list):
    OFH = open(file, 'w')
    for l in list:
        thestr = l + "\n"
        thestr = thestr.encode('utf8')
        OFH.write(thestr)
    OFH.close()

def write_kvfile_fromlist(file, list, delim=' '):
    OFH = open(file, 'w')
    for l in list:
        for i in range(0,len(l)):
            l[i] = re.sub("\s", "____", l[i])
        thestr = delim.join(l) + "\n"
        thestr = thestr.encode('utf8')
        OFH.write(thestr)
    OFH.close()

def write_kvfile_fromdict(file, indict):
    dict = indict.copy()
    OFH = open(file, 'w')
    for k in dict.keys():
        if not dict[k]:
            dict[k] = "none"
        cleank = re.sub("\s+", "____", k)
        thestr = ' '.join([cleank, dict[k], '\n'])
        thestr = thestr.encode('utf8')
        OFH.write(thestr)
    OFH.close()

def touch_file(file):
    return(open(file, 'a').close())

def run_command_in_container(image=None, cmd="echo HELLO WORLD", fileget=None, fileput=None):
    if not image or not cmd:
        raise Exception("Invalid input: image="+str(image)+" cmd="+str(cmd))

    try:
        imageId = discover_imageId(image)
    except Exception as err:
        print str(err)
        return(list())

    olines = list()
    fbuf = ""

    try:
        docker_cli = contexts['docker_cli']
        
        container = docker_cli.create_container(image=image, command="/bin/bash -c '"+cmd+"'", tty=False)

        docker_cli.create_container(image=image, command="/bin/bash -c '"+cmd+"'", tty=False)
        if fileput:
            try:
                TFH=open(fileput, 'r')
                dat = TFH.read()
                TFH.close()
                docker_cli.put_archive(container.get('Id'), "/", dat)
            except Exception as err:
                traceback.print_exc()
                print str(err)
                pass
        response = docker_cli.start(container=container.get('Id'))
        output = docker_cli.logs(container=container.get('Id'), stdout=True, stderr=True, stream=True)
        for l in output:
            olines.append(l)

        if fileget:
            try:
                tstream,stat = docker_cli.get_archive(container, fileget)
                TFH = io.BytesIO(tstream.data)
                tar=tarfile.open(fileobj=TFH, mode='r', format=tarfile.PAX_FORMAT)
                for member in tar.getmembers():
                    fbuf = tar.extractfile(member).read()
                tar.close()
                TFH.close()
            except Exception as err:
                fbuf = ""
                pass

    except Exception as err:
        raise err
    finally:
        try:
            docker_cli.remove_container(container=container.get('Id'), force=True)
        except:
            pass

    return(olines, fbuf)

def get_files_from_tarfile(intarfile):
    allfiles = {}

    try:
        tar = tarfile.open(intarfile)
        for member in tar.getmembers():
            finfo = {}
            finfo['name'] = re.sub("^\./", "/", member.name.decode('utf8'))
            finfo['fullpath'] = os.path.normpath(finfo['name'])
            finfo['size'] = member.size
            finfo['mode'] = member.mode

            finfo['linkdst'] = None
            if member.isfile():
                finfo['type'] = 'file'
            elif member.isdir():
                finfo['type'] = 'dir'
            elif member.issym():
                finfo['type'] = 'slink'
                finfo['linkdst'] = re.sub("^\./", "/", member.linkname.decode('utf8'))
            elif member.islnk():
                finfo['type'] = 'hlink'
                finfo['linkdst'] = re.sub("^\./", "/", member.linkname.decode('utf8'))
            elif member.isdev():
                finfo['type'] = 'dev'
            else:
                finfo['type'] = 'UNKNOWN'

            if finfo['type'] == 'slink' or finfo['type'] == 'hlink':
                if re.match("^/", finfo['linkdst']):
                    fullpath = finfo['linkdst']
                else:
                    dstlist = finfo['linkdst'].split('/')
                    srclist = finfo['name'].split('/')
                    srcpath = srclist[0:-1]
                    fullpath = '/'.join(srcpath + dstlist)
                    fullpath = os.path.normpath('/'.join(srcpath + dstlist))
                finfo['fullpath'] = fullpath

            allfiles[finfo['name']] = finfo

        tar.close()
    except:
        pass

    return(allfiles)

def get_files_from_path(inpath):
    filemap = {}
    allfiles = {}
    real_root = os.open('/', os.O_RDONLY)

    try:
        os.chroot(inpath)
        #for root, dirs, files in os.walk('/', followlinks=True):
        for root, dirs, files in os.walk('/', followlinks=False):
            for name in dirs + files:
                filename = os.path.join(root, name).decode('utf8')
                osfilename = os.path.join(root, name)

                fstat = os.lstat(osfilename)

                finfo = {}
                finfo['name'] = filename
                finfo['fullpath'] = os.path.normpath(osfilename)
                finfo['size'] = fstat.st_size
                finfo['mode'] = fstat.st_mode
                
                mode = finfo['mode']
                finfo['linkdst'] = None
                finfo['linkdst_fullpath'] = None
                if S_ISREG(mode):
                    finfo['type'] = 'file'
                elif S_ISDIR(mode):
                    finfo['type'] = 'dir'
                elif S_ISLNK(mode):
                    finfo['type'] = 'slink'
                    finfo['linkdst'] = os.readlink(osfilename)
                elif S_ISCHR(mode) or S_ISBLK(mode):
                    finfo['type'] = 'dev'
                else:
                    finfo['type'] = 'UNKNOWN'

                if finfo['type'] == 'slink' or finfo['type'] == 'hlink':
                    if re.match("^/", finfo['linkdst']):
                        fullpath = finfo['linkdst']
                    else:
                        dstlist = finfo['linkdst'].split('/')
                        srclist = finfo['name'].split('/')
                        srcpath = srclist[0:-1]
                        fullpath = os.path.normpath(os.path.join(finfo['linkdst'], osfilename))
                    finfo['linkdst_fullpath'] = fullpath

                fullpath = os.path.realpath(osfilename)

                finfo['othernames'] = {}
                for f in [fullpath, finfo['linkdst_fullpath'], finfo['linkdst'], finfo['name']]:
                    if f:
                        finfo['othernames'][f] = True

                allfiles[finfo['name']] = finfo

        # first pass, set up the basic file map
        for name in allfiles.keys():
            finfo = allfiles[name]
            finfo['othernames'][name] = True

            filemap[name] = finfo['othernames']
            for oname in finfo['othernames']:
                filemap[oname] = finfo['othernames']

        # second pass, include second order
        newfmap = {}
        count = 0
        while newfmap != filemap or count > 5:
            count += 1
            filemap.update(newfmap)
            newfmap.update(filemap)
            for mname in newfmap.keys():
                for oname in newfmap[mname].keys():
                    newfmap[oname].update(newfmap[mname])

    except Exception as err:
        traceback.print_exc()
        print str(err)
        pass
    finally:
        os.fchdir(real_root)
        os.chroot('.')

    return(filemap, allfiles)

def grouper(inlist, chunksize):
    return (inlist[pos:pos + chunksize] for pos in xrange(0, len(inlist), chunksize))




#!/usr/bin/env python

import sys
import os
import json
import re
import rpm
from rpmUtils.miscutils import splitFilename
from anchore import anchore_image
import anchore.anchore_utils

gate_name = "ANCHORESEC"
triggers = {
    'VULNLOW':
    {
        'description':'triggers if a vulnerability of LOW severity is found',
        'params':'none'
    },
    'VULNMEDIUM':
    {
        'description':'triggers if a vulnerability of MED severity is found',
        'params':'none'
    },
    'VULNHIGH':
    {
        'description':'triggers if a vulnerability of HIGH severity is found',
        'params':'none'
    },
    'VULNCRITICAL':
    {
        'description':'triggers if a vulnerability of CRITICAL severity is found',
        'params':'none'
    },
    'VULNUNKNOWN':
    {
        'description':'triggers if a vulnerability of UNKNOWN severity is found',
        'params':'none'
    },
    'UNSUPPORTEDDISTRO':
    {
        'description':'triggers if a vulnerability scan cannot be run against the image due to lack of vulnerability feed data for the images distro',
        'params':'none'
    },
}


try:
    config = anchore.anchore_utils.init_gate_cmdline(sys.argv, gate_name, gate_help=triggers)
except Exception as err:
    print str(err)
    sys.exit(1)

if not config:
    print "ERROR: could not set up environment for gate"
    sys.exit(1)

imgid = config['imgid']

try:
    params = config['params']
except:
    params = None

cvedirroot = '/'.join([config['anchore_config']['image_data_store'], "../cve-data"])

try:
    image = anchore_image.AnchoreImage(imgid, config['anchore_config']['image_data_store'], {})
    cve_data = anchore.anchore_utils.cve_load_data(image)
    report = anchore.anchore_utils.cve_scanimage(cve_data, imgid)
except Exception as err:
    import traceback
    traceback.print_exc()
    print "ERROR: could not scan image for CVEs: " + str(err)
    outlist = list()
    outlist.append("UNSUPPORTEDDISTRO Cannot load CVE data for image distro to perform scan. Message from service: "+str(err))
    anchore.anchore_utils.save_gate_output(imgid, gate_name, outlist)
    sys.exit(0)

outlist = list()
for k in report.keys():
    for cvepkg in report[k]:
        vuln = cvepkg
        cve = k
        pkg = vuln['pkgName']
        sev = vuln['severity']
        url = vuln['url']
        if sev == 'Low':
            t = "VULNLOW"
        elif sev == 'Medium':
            t = "VULNMEDIUM"
        elif sev == "High":
            t = "VULNHIGH"
        elif sev == "Critical":
            t = "VULNCRITICAL"
        else:
            t = "VULNUNKNOWN"

        d = {'id':cve, 'desc':sev + " Vulnerability found in package - " + pkg + " (" + cve + " - " + url + ")"}
        #outlist.append(t + " " + sev + " Vulnerability found in package - " + pkg + " (" + cve + " - " + url + ")")
        outlist.append(t + " " + json.dumps(d))

anchore.anchore_utils.save_gate_output(imgid, gate_name, outlist)

sys.exit(0)



import os
import subprocess
import re
import logging

class ScriptExecutor:
    _logger = logging.getLogger(__name__)

    def __init__(self, path, script_name, suffix_list=['', '.py', '.sh'], path_overrides=[]):
        self.path = path
        self.script_name = script_name
        self.suffix_list = suffix_list
        self.path_overrides = path_overrides
        try:
            self.check()
        except Exception as err:
            raise err

    def check(self):
        allpaths = self.path_overrides + [self.path]

        match=False
        matchcmd = None 
        for p in allpaths:
            for cmd in [ '/'.join([p, self.script_name]) + x for x in self.suffix_list ]:
                if os.path.exists(cmd) and os.access(cmd, os.R_OK ^ os.X_OK): 
                    if not match:
                        match=True
                        matchcmd = cmd
                    else:
                        # ambiguous
                        raise ValueError("input script is ambiguous: match cmd=" + str(cmd) + " previous match="+str(matchcmd))
                if matchcmd:
                    continue
                
        if matchcmd:
            self.thecmd = matchcmd
        else:
            raise Exception("cannot locate executable input script: " + self.script_name + "("+'|'.join(self.suffix_list)+")")

        return(True)
        
    def execute(self, capture_output=False, cmdline=None, **kwargs):
        output = list()
        script = self.thecmd
        if not os.access(script, os.R_OK ^ os.X_OK) or re.match(".*~$", script):
            raise Exception("Command not executable or is invalid filename: " + script)

        if cmdline:
            cmd = [script] + cmdline.split()
        else:
            cmd = [script]

        self._logger.debug('Executing script: %s' % ' '.join(cmd))

        if capture_output:
            try:
                output = (' '.join(cmd), 0, subprocess.check_output(cmd, stderr=subprocess.STDOUT, **kwargs))
            except subprocess.CalledProcessError as e:
                output = (' '.join(cmd), e.returncode, e.output)
        else:
            output = (' '.join(cmd), subprocess.call(cmd, **kwargs), None)

        self._logger.debug('%s return code %d' % (output[0], output[1]))
        if output[2]:
            self._logger.debug('%s output:' % (script))
            for l in output[2].splitlines():
                l = l.strip()
                self._logger.debug('%s | %s' % (script, l))

        return(output)

class ScriptSetExecutor:
    """
    A script module is a module for executing a set of user-provided scripts found in a specific directory in
    lexicographically sorted order

    """
    _logger = logging.getLogger(__name__)

    def __init__(self, path, file_prefix='', ascending=True, suffix_list=['', '.sh', '.py'], path_overrides=[]):
        assert path is not None

        self.inputdir = path
        self.prefix = file_prefix
        self.sort_ascending = ascending
        self.path_overrides = path_overrides
        self.suffix_list = suffix_list
        self.allpaths = list()
        try:
            self.check()
        except Exception as err:
            raise err

    def check(self, init_if_missing=False):
        """
        Check the path and construct if not found and init_if_missing=True
        :param init_if_missing: create the path if not found
        :return: true if exists false if not
        """
        for d in self.path_overrides + [self.inputdir]:            
            if os.path.exists(d):
                self.allpaths.append(d)
        if len(self.allpaths) > 0:
            return(True)

        if init_if_missing:
            os.makedirs(self.inputdir)
            return True

    def get_scripts(self):
        """
        Get the scripts at the path in sorted order as set in the module properties
        :return: a sorted list of scripts
        """
        ret = list()
        for d in self.allpaths:
            scripts = filter(lambda x: x.startswith(self.prefix), os.listdir(d))
            scripts.sort(reverse=(not self.sort_ascending))
            ret = ret + [os.path.join(d, x) for x in scripts]

        return(ret)

    def csums(self):
        ret = {}
        scripts = self.get_scripts()
        for script in scripts:
            try:
                import hashlib
                FH=open(script, 'r')
                csum = hashlib.md5(FH.read()).hexdigest()
                FH.close()
            except:
                csum = "N/A"
            ret[script] = csum
        return(ret)

    def execute(self, capture_output=False, fail_fast=False, cmdline=None, lastcsums=None, **kwargs):
        """
        Pass in the kwargs as --<name> <value> pairs.
        :param capture_output: if True then return output of script in return dict. If False, only return code
        :param fail_fast:if True then quit execution after first non-zero return code, else execute all
        :returns list of (script, returncode, stdout) tuples in order of execution
        """

        currcsums = self.csums()
        
        scripts = self.get_scripts()
        output = []

        for script in scripts:
            if not os.access(script, os.R_OK ^ os.X_OK) or re.match(".*~$", script):
                # Skip any that are not executable (e.g. README or txt files)
                continue

            if lastcsums:
                if (script in currcsums and script in lastcsums) and currcsums[script] == lastcsums[script]:
                    # skip if the analyzer has not changed since the last time it was run
                    self._logger.debug("Skipping analyzer %s since this analyzer version has already executed in the past" % script)
                    continue

            self._logger.debug('Executing script: %s' % script)

            try:
                csum = currcsums[script]
            except:
                csum = "N/A"

            if cmdline:
                cmd = [script] + cmdline.split()
            else:
                cmd = [script]

            if capture_output:
                try:
                    outstr = subprocess.check_output(cmd, stderr=subprocess.STDOUT, **kwargs)
                    outstr = outstr.decode('utf8')
                    #outstr = rawstr.encode('utf8')
                    output.append((' '.join(cmd), 0, outstr))
                except subprocess.CalledProcessError as e:
                    outstr = e.output.decode('utf8')
                    #outstr = rawstr.encode('utf8')
                    output.append((' '.join(cmd), e.returncode, outstr))
            else:
                output.append((' '.join(cmd), subprocess.call(cmd, **kwargs), None))

            self._logger.debug('%s return code %d' % (output[-1][0], output[-1][1]))
            if output[-1][2]:
                self._logger.debug('%s output:' % (script))
                for l in output[-1][2].splitlines():
                    l = l.strip()
                    self._logger.debug('%s | %s' % (script, l))

            if fail_fast and output[-1][1]:
                break

        return output

